################################################################################
# Project   : CorSight2 - Custom Module LUT Design
# Title     : Vivado FPGA simulation script file
# Copyright (c) 2017 NET New Electronic Technology GmbH
################################################################################

# Set the relative path to the OpenCamera home directory
set OpenCamHome "../../../.."

# Define Cs2OpenCamCfg default values
set Cs2OpenCamCfg(OpenCamDir)  "Undefined"
set Cs2OpenCamCfg(CompileDir)  "Undefined"
set Cs2OpenCamCfg(ReleaseDir)  "Undefined"

# Read customer defined environment variables
if {[file exist $OpenCamHome/CompileTools/Set_Customer_EnvVar.tcl] == 1} {
    source $OpenCamHome/CompileTools/Set_Customer_EnvVar.tcl
} else {
    puts "\n-- \[Compile_OpenCam_FPGA.tcl\] ERROR - OpenCamera environment file \"Set_Customer_EnvVar.tcl\" not found - Compile aborted\n"
    return error
}

# Attempt to set local compile variables from the predefined computer environment script.
if {[file exist $OpenCamHome/CompileTools/Set_PreDef_EnvVar.tcl] == 1 && [file isdirectory $Cs2OpenCamCfg(OpenCamDir)] == 0} {
    source $OpenCamHome/CompileTools/Set_PreDef_EnvVar.tcl
}

# If the OpenCamera source directory still can not be found print an error message and abort the compilation.
if {[file isdirectory $Cs2OpenCamCfg(OpenCamDir)] == 0} {
    puts "\n-- \[Compile_OpenCam_FPGA.tcl\] ERROR - OpenCamera source file directory \"$Cs2OpenCamCfg(OpenCamDir)\" not found - Compile aborted\n"
    return error
}

################################################################################

set SourceDir     "$Cs2OpenCamCfg(OpenCamDir)/src"
set TestBenchDir  "$Cs2OpenCamCfg(OpenCamDir)/sim/TestBench"
set SimulationDir "$Cs2OpenCamCfg(OpenCamDir)/sim/Simulation/Custom_LutDesign/Xsim"

################################################################################

puts ""
puts "========  Compiling CorSight2 LUT Design Testbench  ========"
puts ""

start_gui
create_project -force Custom_LutDesign "$SimulationDir" -part xc7a75tfgg484-2

set_property target_language VHDL [current_project]
set_property simulator_language VHDL [current_project]
set_property target_simulator XSIM [current_project]

################################################################################

read_vhdl "$SourceDir/Wrapper/CustSimPkg.vhd"
read_vhdl "$SourceDir/Wrapper/CustomPkg.vhd"
read_vhdl "$SourceDir/Wrapper/DebugModule.vhd"

read_vhdl "$SourceDir/LutDesign/LutVerPkg.vhd"
read_vhdl "$SourceDir/LutDesign/LutPkg.vhd"
read_vhdl "$SourceDir/LutDesign/LutSysBusIf.vhd"
read_vhdl "$SourceDir/LutDesign/LutRAM.vhd"
read_vhdl "$SourceDir/LutDesign/LutCtrl.vhd"
read_vhdl "$SourceDir/LutDesign/LutModule.vhd"

read_vhdl "$TestBenchDir/SimCfg_Xsim.vhd"
read_vhdl "$TestBenchDir/CustomCfg_LutDesign.vhd"
read_vhdl "$TestBenchDir/CustomCfgLib.vhd"
read_vhdl "$TestBenchDir/CustomSimLib.vhd"
read_vhdl "$TestBenchDir/CustomFctLib.vhd"
read_vhdl "$TestBenchDir/SimParaInit.vhd"
read_vhdl "$TestBenchDir/ImgBusAnalyzer.vhd"
read_vhdl "$TestBenchDir/ImgBusStimulus.vhd"
read_vhdl "$TestBenchDir/SysBusStimulus.vhd"
read_vhdl "$TestBenchDir/CustomModuleTB.vhd"

################################################################################

set_property -name {xsim.compile.xvhdl.nosort} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.compile.xvlog.nosort} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.compile.xvlog.relax} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.compile.xvhdl.relax} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.compile.incremental} -value {false} -objects [get_filesets sim_1]

set_property -name {xsim.elaborate.relax} -value {false} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.load_glbl} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.debug_level} -value {all} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.mt_level} -value {auto} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.rangecheck} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.sdf_delay} -value {sdfmax} -objects [get_filesets sim_1]

set_property -name {xsim.simulate.runtime} -value {10ms} -objects [get_filesets sim_1]
set_property SOURCE_SET sources_1 [get_filesets sim_1]

set_property top CustomTB [get_filesets sim_1]
set_property top_lib xil_defaultlib [get_filesets sim_1]

set_msg_config -suppress -id {Wavedata 42-489}

launch_simulation
set_property display_limit 70000 [current_wave_config]
add_wave {{/CustomTB}}
