################################################################################
# Project   : CorSight2 - Custom Module MemTest
# Title     : Vivado FPGA simulation script file
# Copyright (c) 2017 NET New Electronic Technology GmbH
################################################################################

# Set the relative path to the OpenCamera home directory
set OpenCamHome "../../../.."

# Define Cs2OpenCamCfg default values
set Cs2OpenCamCfg(OpenCamDir)  "Undefined"
set Cs2OpenCamCfg(CompileDir)  "Undefined"
set Cs2OpenCamCfg(ReleaseDir)  "Undefined"

# Read customer defined environment variables
if {[file exist $OpenCamHome/CompileTools/Set_Customer_EnvVar.tcl] == 1} {
    source $OpenCamHome/CompileTools/Set_Customer_EnvVar.tcl
} else {
    puts "\n-- \[Compile_OpenCam_FPGA.tcl\] ERROR - OpenCamera environment file \"Set_Customer_EnvVar.tcl\" not found - Compile aborted\n"
    return error
}

# Attempt to set local compile variables from the predefined computer environment script.
if {[file exist $OpenCamHome/CompileTools/Set_PreDef_EnvVar.tcl] == 1 && [file isdirectory $Cs2OpenCamCfg(OpenCamDir)] == 0} {
    source $OpenCamHome/CompileTools/Set_PreDef_EnvVar.tcl
}

# If the OpenCamera source directory still can not be found print an error message and abort the compilation.
if {[file isdirectory $Cs2OpenCamCfg(OpenCamDir)] == 0} {
    puts "\n-- \[Compile_OpenCam_FPGA.tcl\] ERROR - OpenCamera source file directory \"$Cs2OpenCamCfg(OpenCamDir)\" not found - Compile aborted\n"
    return error
}

################################################################################

set SourceDir     "$Cs2OpenCamCfg(OpenCamDir)/src"
set TestBenchDir  "$Cs2OpenCamCfg(OpenCamDir)/sim/TestBench"
set SimulationDir "$Cs2OpenCamCfg(OpenCamDir)/sim/Simulation/Custom_MemTest/Xsim"

################################################################################

puts ""
puts "========  Compiling CorSight2 MemTest Testbench  ========"
puts ""

start_gui
create_project -force Custom_MemTest "$SimulationDir" -part xc7a75tfgg484-2

set_property target_language VHDL [current_project]
set_property simulator_language VHDL [current_project]
set_property target_simulator XSIM [current_project]

################################################################################

read_vhdl "$SourceDir/Wrapper/CustSimPkg.vhd"
read_vhdl "$SourceDir/Wrapper/CustomPkg.vhd"
read_vhdl "$SourceDir/Wrapper/DebugModule.vhd"

read_vhdl "$SourceDir/MemTest/MemTestVerPkg.vhd"
read_vhdl "$SourceDir/MemTest/MemTestPkg.vhd"
read_vhdl "$SourceDir/MemTest/MemTestSysIf.vhd"
read_vhdl "$SourceDir/MemTest/MemTestCtrl.vhd"
read_vhdl "$SourceDir/MemTest/MemTestModule.vhd"

read_vhdl "$TestBenchDir/SimCfg_Xsim.vhd"
read_vhdl "$TestBenchDir/CustomCfg_MemTest.vhd"
read_vhdl "$TestBenchDir/CustomCfgLib.vhd"
read_vhdl "$TestBenchDir/CustomSimLib.vhd"
read_vhdl "$TestBenchDir/CustomFctLib.vhd"
read_vhdl "$TestBenchDir/SimParaInit.vhd"
read_vhdl "$TestBenchDir/ImgBusAnalyzer.vhd"
read_vhdl "$TestBenchDir/ImgBusStimulus.vhd"
read_vhdl "$TestBenchDir/SysBusStimulus.vhd"
read_vhdl "$TestBenchDir/CustomMcbInterface.vhd"
read_vhdl "$TestBenchDir/CustomRAM.vhd"
read_vhdl "$TestBenchDir/CustomModuleTB.vhd"

read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/clocking/mig_7series_v4_0_clk_ibuf.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/clocking/mig_7series_v4_0_infrastructure.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/clocking/mig_7series_v4_0_iodelay_ctrl.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/clocking/mig_7series_v4_0_tempmon.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_arb_mux.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_arb_row_col.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_arb_select.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_bank_cntrl.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_bank_common.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_bank_compare.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_bank_mach.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_bank_queue.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_bank_state.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_col_mach.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_mc.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_rank_cntrl.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_rank_common.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_rank_mach.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/controller/mig_7series_v4_0_round_robin_arb.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ecc/mig_7series_v4_0_ecc_buf.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ecc/mig_7series_v4_0_ecc_dec_fix.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ecc/mig_7series_v4_0_ecc_gen.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ecc/mig_7series_v4_0_ecc_merge_enc.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ecc/mig_7series_v4_0_fi_xor.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ip_top/mig_7series_v4_0_memc_ui_top_std.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ip_top/mig_7series_v4_0_mem_intfc.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_byte_group_io.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_byte_lane.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_calib_top.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_if_post_fifo.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_mc_phy.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_mc_phy_wrapper.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_of_pre_fifo.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_4lanes.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_ck_addr_cmd_delay.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_dqs_found_cal.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_dqs_found_cal_hr.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_init.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_ocd_cntlr.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_ocd_data.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_ocd_edge.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_ocd_lim.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_ocd_mux.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_ocd_po_cntlr.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_ocd_samp.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_oclkdelay_cal.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_prbs_rdlvl.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_rdlvl.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_tempmon.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_wrcal.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_wrlvl.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_wrlvl_off_delay.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_prbs_gen.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_poc_cc.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_poc_edge_store.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_poc_meta.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_poc_pd.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_poc_tap_base.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_poc_top.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_skip_calib_tap.v"
read_vhdl    "$TestBenchDir/MCB_SIM/MT41K64M16XX107/phy/mig_7series_v4_0_ddr_phy_top.vhd"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ui/mig_7series_v4_0_ui_cmd.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ui/mig_7series_v4_0_ui_rd_data.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ui/mig_7series_v4_0_ui_top.v"
read_verilog "$TestBenchDir/MCB_SIM/MT41K64M16XX107/ui/mig_7series_v4_0_ui_wr_data.v"
read_vhdl    "$TestBenchDir/MCB_SIM/MT41K64M16XX107/MCB_SIM_MT41K64M16XX107_mig_sim.vhd"
read_vhdl    "$TestBenchDir/MCB_SIM/MT41K64M16XX107/MCB_SIM_MT41K64M16XX107.vhd"

read_verilog -sv "$TestBenchDir/DDR3-RAM_1Gbit/ddr3_1Gbit_model_parameters.vh"
read_verilog -sv "$TestBenchDir/DDR3-RAM_1Gbit/ddr3_1Gbit_model.sv"

################################################################################

set_property -name {xsim.compile.xvhdl.nosort} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.compile.xvlog.nosort} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.compile.xvlog.relax} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.compile.xvhdl.relax} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.compile.incremental} -value {false} -objects [get_filesets sim_1]

set_property -name {xsim.elaborate.relax} -value {false} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.load_glbl} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.debug_level} -value {all} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.mt_level} -value {auto} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.rangecheck} -value {true} -objects [get_filesets sim_1]
set_property -name {xsim.elaborate.sdf_delay} -value {sdfmax} -objects [get_filesets sim_1]

set_property -name {xsim.simulate.runtime} -value {15ms} -objects [get_filesets sim_1]
set_property SOURCE_SET sources_1 [get_filesets sim_1]

set_property top CustomTB [get_filesets sim_1]
set_property top_lib xil_defaultlib [get_filesets sim_1]

set_msg_config -suppress -id {Wavedata 42-489}

launch_simulation
add_wave {{/CustomTB}}
