//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : CustomModule
// Submodules : cmif2sif
//              siftop
//
// Purpose    : CustomModule Top Level
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps



module CustomModule (
    // Module Infrastructure Signals
    ImgRstxRI,
    SysClkxCI,
    SysRstxRI,

    // ImageBus Receive Port
    ImgRxPixDataxDI,
    ImgRxPixRdyxMO,
    ImgRxPixStatxSI,
    ImgRxPixValxMI,

    // ImageBus Transmit Port
    ImgTxPixDataxDO,
    ImgTxPixRdyxMI,
    ImgTxPixStatxSO,
    ImgTxPixValxMO,

    // SystemBus Control Interface
    SysAckxMO,
    SysAddrxAI,
    SysErrxMO,
    SysRdDataxDO,
    SysReqxMI,
    SysWrDataxDI,
    SysWrEnxMI,
	
	// Custom Module GPIO
    CustIoCtrlxMO,
    CustIoDataOutxDO,
    CustIoDataInxDI,

    // command
    px_cmd_addr_o,
    px_cmd_bl_o,
    px_cmd_en_o,
    px_cmd_full_i,
    px_cmd_instr_o,

    // read
    px_rd_data_i,
    px_rd_empty_i,
    px_rd_en_o,

    // write
    px_wr_data_o,
    px_wr_en_o,
    px_wr_full_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // Module Infrastructure Signals
  //--------------------------------------------------------
  input           ImgRstxRI;            // ImagePipeline Reset
  input           SysClkxCI;            // Custom Module Clock
  input           SysRstxRI;            // Custom Module Reset

  //--------------------------------------------------------
  // Image Receive Port
  //--------------------------------------------------------
  input  [35:0]   ImgRxPixDataxDI;      // 3x12 bit RGB pixel data
  output          ImgRxPixRdyxMO;       // receive pixel ready - not implemented in GigEpro
  input  [2:0]    ImgRxPixStatxSI;      // image frame sync code
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-single-Line-Frame
                                        // (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)
  input           ImgRxPixValxMI;       // data valid

  //--------------------------------------------------------
  // Image Transmit Port
  //--------------------------------------------------------
  output [35:0]   ImgTxPixDataxDO;      // packed data
  input           ImgTxPixRdyxMI;       // ImageBus Transmit Pixel Ready -
                                        // ignore if cIMG_HANDSHAKE_EN = False
  output [2:0]    ImgTxPixStatxSO;      // 000 No-Status (NONE)
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-single-Line-Frame
                                        // (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)
  output          ImgTxPixValxMO;       // transaction valid

  //--------------------------------------------------------
  // Control Bus Port
  //--------------------------------------------------------
  output          SysAckxMO;            // indicates successful cycle
                                        // termination
  input  [16:0]   SysAddrxAI;           // address input array
  output          SysErrxMO;            // indicates abnormal cycle termination
  output [31:0]   SysRdDataxDO;         // read data
  input           SysReqxMI;            // cycle input indicating a valid bus
                                        // cycle
  input  [31:0]   SysWrDataxDI;         // write data
  input           SysWrEnxMI;           // write enable
  
  //--------------------------------------------------------
  // Custom Module GPIO
  //--------------------------------------------------------
  output [5:0]    CustIoCtrlxMO;        // GPIO Output Control
                                        // 00: CustIoDataOutxDO(1) disable, CustIoDataOutxDO(0) disable
                                        // 01: CustIoDataOutxDO(1) disable, CustIoDataOutxDO(0) enable
                                        // 10: CustIoDataOutxDO(1) enable,  CustIoDataOutxDO(0) disable
                                        // 11: CustIoDataOutxDO(1) enable,  CustIoDataOutxDO(0) enable
  output [5:0]    CustIoDataOutxDO;     // GPIO Output Data
  input  [5:0]    CustIoDataInxDI;      // GPIO Input Data (aktiv Low)

  //--------------------------------------------------------
  // command
  //--------------------------------------------------------
  output [29:0]   px_cmd_addr_o;        // Byte start address for current
                                        // transaction. Addresses must be
                                        // aligned to port size:
                                        // 32-bit ports: Lower two bits must be
                                        //  0s.
  output [5:0]    px_cmd_bl_o;          // Burst length in number of user words
                                        //  for the current transaction. Burst
                                        // length is encoded as 0 to 63,
                                        // representing 1 to 64 user words [for
                                        //  example, b00011 is a burst length 4
                                        //  transaction]. The user word width
                                        // equals the port width [for example,
                                        // a burst length of 3 on a 64-bit port
                                        //  transfers 3 x 64-bit user words =
                                        // 192 bits total].
  output          px_cmd_en_o;          // This active-High signal is the
                                        // write-enable signal for the Command
                                        // FIFO. This signal is covered in more
                                        //  detail in Chapter 4.
  input           px_cmd_full_i;        // This active-High output is the full
                                        // flag for the Command FIFO. It
                                        // indicates the FIFO cannot accept any
                                        //  more commands and blocks writes to
                                        // the Command FIFO.
  output [2:0]    px_cmd_instr_o;       // Command code for the current
                                        // instruction. Bit 0 represents the
                                        // READ/WRITE select, Bit 1 is Auto
                                        // Precharge enable, and Bit 2
                                        // represents Refresh, which always
                                        // takes priority:
                                        // Write: 3b000
                                        // Read: 3b001
                                        // This signal is covered in more
                                        // detail in Chapter 4.

  //--------------------------------------------------------
  // read
  //--------------------------------------------------------
  input  [31:0]   px_rd_data_i;         // Read Data value returning from
                                        // memory. This signal is driven by the
                                        //  output of the Read Data FIFO into
                                        // FPGA logic. PX_SIZE can be 32, 64,
                                        // or 128 bits, depending on the port
                                        // configuration.
  input           px_rd_empty_i;        // This active-High signal is the empty
                                        //  flag for the Read Data FIFO. It
                                        // indicates there is no valid data in
                                        // the FIFO.
  output          px_rd_en_o;           // This active-High signal is the read
                                        // enable for the Read Data FIFO. Read
                                        // Data is clocked out of the FIFO on
                                        // the rising edge of pX_rd_clk when
                                        // pX_rd_en = 1 and pX_rd_empty = 0.

  //--------------------------------------------------------
  // write
  //--------------------------------------------------------
  output [31:0]   px_wr_data_o;         // Write Data value to be loaded into
                                        // Write Data FIFO and sent to memory.
                                        // PX_SIZE can be 32, 64, or 128 bits,
                                        // depending on port configuration.
  output          px_wr_en_o;           // This active-High signal is the write
                                        //  enable for the Write Data FIFO. It
                                        // indicates that the value on
                                        // pX_wr_data is valid for loading into
                                        //  the FIFO. Data is loaded on the
                                        // rising edge of pX_wr_clk when
                                        // pX_wr_en = 1 and pX_wr_full = 0.
  input           px_wr_full_i;         // This active-High signal is the full
                                        // flag for the Write Data FIFO. When
                                        // this signal is high, it prevents
                                        // data from being loade into the FIFO.
  
  
  
  
  reg [35:0] ImgTxPixDataxReg;             
  reg [2:0]  ImgTxPixStatxReg;
  reg        ImgTxPixValxReg;
  
  // --- parameters ---
  reg  paramEnable;   
  reg  [2:0] paramSpeed;   
  reg  [2:0] paramSize;   
  reg  paramColor;

  reg [31:0] cbReadOut;  
  wire blockSel;
  wire [14:0] regId;
  reg  AckReg;
  
  reg    [11:0]   vcnt;
  reg    [11:0]   hcnt;
  reg    [31:0]   fcnt;
  reg    [31:0]   msgcnt;
  reg    [31:0]   msgdata1;
  reg    [31:0]   msgdata2;
  
  reg    [11:0]   vmax;
  reg    [11:0]   hmax;
  
  reg    [11:0]   ball1vpos;
  reg    [11:0]   ball1hpos;
  reg    ball1dirRight;
  reg    ball1dirDown;
  reg    [6:0]  ball1PatternHor;
  reg    [6:0]  ball1PatternVert;
  reg           ball1HorEnable;
  reg           ball1VertEnable;
  
  reg    [11:0]   ball2vpos;
  reg    [11:0]   ball2hpos;
  reg    ball2dirRight;
  reg    ball2dirDown;
  reg    [6:0]  ball2PatternHor;
  reg    [6:0]  ball2PatternVert;
  reg           ball2HorEnable;
  reg           ball2VertEnable;
  
  reg    [11:0]   ball3vpos;
  reg    [11:0]   ball3hpos;
  reg    ball3dirRight;
  reg    ball3dirDown;
  reg    [6:0]  ball3PatternHor;
  reg    [6:0]  ball3PatternVert;
  reg           ball3HorEnable;
  reg           ball3VertEnable;
  
  
  wire startOfFrame;
  wire startOfLine;
  wire startOfLineInput;
  wire endOfLine;
  wire startOfLastLine;
  
  reg [5:0] posInc;
  reg [6:0] patMax;
  
  assign startOfFrame     = (ImgRxPixStatxSI==3'b001)? 1 : 0;
  assign startOfLineInput = (ImgRxPixStatxSI==3'b100)? 1 : 0;
  assign endOfLine        = (ImgRxPixStatxSI==3'b110)? 1 : 0;
  assign startOfLastLine  = (ImgRxPixStatxSI==3'b101)? 1 : 0;
  
  assign startOfLine = startOfLineInput|startOfLastLine;
  
   
  always @ (paramSpeed ) begin
    posInc <= 1;
    case (paramSpeed)
        3'h0 : posInc <=  1;
        3'h1 : posInc <=  2;
        3'h2 : posInc <=  4;
		3'h3 : posInc <=  8;
		3'h4 : posInc <= 16;
		3'h5 : posInc <= 32;
		3'h6 : posInc <= 42;
		3'h7 : posInc <= 63;
    endcase
  end
  
  always @ (paramSize ) begin
    patMax <= 15;
    case (paramSize)
        3'h0 : patMax <= 15;
        3'h1 : patMax <= 30;
        3'h2 : patMax <= 40;
		3'h3 : patMax <= 50;
		3'h4 : patMax <= 60;
		3'h5 : patMax <= 70;
		3'h6 : patMax <= 90;
		3'h7 : patMax <= 127;
    endcase
  end
  
  
  
  
  
  always @(posedge SysClkxCI ) begin
    if (paramEnable) begin
	  if (ball1HorEnable && ball1VertEnable) begin
	      if (paramColor) ImgTxPixDataxReg  <= 36'b000000000000111111111111000000000000;
		  else            ImgTxPixDataxReg  <= 36'b111111111111111111111111111111111111;
      end else if (ball2HorEnable && ball2VertEnable) begin
	      if (paramColor) ImgTxPixDataxReg  <= 36'b000000000000000000000000111111111111;
		  else            ImgTxPixDataxReg  <= 36'b111111111111111111111111111111111111;
      end else if (ball3HorEnable && ball3VertEnable) begin
	      if (paramColor) ImgTxPixDataxReg  <= 36'b111111111111000000000000000000000000;
		  else            ImgTxPixDataxReg  <= 36'b111111111111111111111111111111111111;
	  end else begin
   	      ImgTxPixDataxReg  <= ImgRxPixDataxDI;
	  end
	         
	end else begin
	   ImgTxPixDataxReg  <= ImgRxPixDataxDI;
	end
	
	ImgTxPixStatxReg  <= ImgRxPixStatxSI;          
    ImgTxPixValxReg   <= ImgRxPixValxMI;  
  end
  
  assign blockSel = (SysAddrxAI[16:4]!=13'b1010000000000);
  assign regId = SysAddrxAI[16:2];
  
  
  // --------------------------------------------------
  //               counters 
   // --------------------------------------------------
   
  always @(posedge SysClkxCI ) begin
    if (ImgRxPixValxMI) begin
      if (startOfFrame) fcnt <= fcnt + 1'b1;
    end
  end
  
  always @(posedge SysClkxCI ) begin
   
	if (ImgRxPixValxMI) begin
	
	  if (startOfLine) begin
	    hcnt <= 1'b000000000000;
      end
      else begin
        hcnt <= hcnt + 12'b000000000001;
      end
	  
	end
  end
  
  always @(posedge SysClkxCI ) begin
   
	if (ImgRxPixValxMI) begin
	
	  if (startOfFrame) begin
	    vcnt <= 12'b000000000000;
      end
      else if (startOfLine) begin
        vcnt <= vcnt + 12'b000000000001;
      end
	  
	end
  end
  
  always @(posedge SysClkxCI ) begin
    if (ImgRxPixValxMI) begin
	  if (endOfLine)       hmax <= hcnt-{0,patMax};
	  if (startOfLastLine) vmax <= vcnt-{0,patMax};
	end
  end
  
  always @(posedge SysClkxCI ) begin
   if (ImgRxPixValxMI) begin
     if (startOfFrame) begin 
        		
		if (ball1dirDown) 
		  ball1vpos <= ball1vpos + posInc;
		else 
		  ball1vpos <= ball1vpos - {0,posInc};
		
		if (ball2dirDown) 
		  ball2vpos <= ball2vpos + posInc;
		else 
		  ball2vpos <= ball2vpos - {0,posInc};
		  
		if (ball3dirDown) 
		  ball3vpos <= ball3vpos + posInc;
		else 
		  ball3vpos <= ball3vpos - {0,posInc};
		   
	    if ( ball1vpos>vmax) begin 
		  msgcnt <= msgcnt +  32'b00000000000000000000000000000001;
		  msgdata1 <= msgcnt;
          msgdata2 <= fcnt;	
		  if (ball1dirDown) begin
		    ball1dirDown <= 0;
			ball1vpos <= vmax;	
            	
		  end else begin
		    ball1dirDown <= 1;
			ball1vpos    <= 0;			
		  end
		  
		end
		
		if ( ball2vpos>vmax) begin 
		  msgcnt <= msgcnt +  32'b00000000000000000000000000000001;
		  msgdata1 <= msgcnt;
          msgdata2 <= fcnt;	
		  if (ball2dirDown) begin
		    ball2dirDown <= 0;
			ball2vpos <= vmax;			
		  end else begin
		    ball2dirDown <= 1;
			ball2vpos    <= 0;
		  end
		  
		end
		
		if ( ball3vpos>vmax) begin 
		  msgcnt <= msgcnt +  32'b00000000000000000000000000000001;
		  msgdata1 <= msgcnt;
          msgdata2 <= fcnt;	
		  if (ball3dirDown) begin
		    ball3dirDown <= 0;
			ball3vpos <= vmax;
		  end else begin
		    ball3dirDown <= 1;
			ball3vpos    <= 0;
		  end
		  
		end
		
		if (ball1dirRight) begin 
		  ball1hpos <= ball1hpos + posInc;
		end else begin
		  ball1hpos <= ball1hpos - {0,posInc};
		end
		
		if (ball2dirRight) begin 
		  ball2hpos <= ball2hpos + posInc;
		end else begin
		  ball2hpos <= ball2hpos - {0,posInc};
		end
		
		if (ball3dirRight) begin 
		  ball3hpos <= ball3hpos + posInc;
		end else begin
		  ball3hpos <= ball3hpos - {0,posInc};
		end
		   
	    if ( ball1hpos>hmax) begin 
		  msgcnt <= msgcnt +  32'b00000000000000000000000000000001;
		  msgdata1 <= msgcnt;
          msgdata2 <= fcnt;	
		  if (ball1dirRight) begin
		    ball1dirRight <= 0;
			ball1hpos <= hmax;
		  end else begin
		    ball1dirRight <= 1;
			ball1hpos    <= 0;
		  end
		  
		end
		
		if ( ball2hpos>hmax) begin 
		  msgcnt <= msgcnt +  32'b00000000000000000000000000000001;
		  msgdata1 <= msgcnt;
          msgdata2 <= fcnt;	
		  if (ball2dirRight) begin
		    ball2dirRight <= 0;
			ball2hpos <= hmax;
		  end else begin
		    ball2dirRight <= 1;
			ball2hpos    <= 0;
		  end
		  
		end
		
		if ( ball3hpos>hmax) begin 
		  msgcnt <= msgcnt +  32'b00000000000000000000000000000001;
		  msgdata1 <= msgcnt;
          msgdata2 <= fcnt;	
		  if (ball3dirRight) begin
		    ball3dirRight <= 0;
			ball3hpos <= hmax;
		  end else begin
		    ball3dirRight <= 1;
			ball3hpos    <= 0;
		  end
		  
		end
		
	 end
	
  end
  
  if (paramEnable==0) begin
    ball1vpos <= {0,vmax[11:1]};
    ball1hpos <= {0,hmax[11:1]};;
    ball1dirRight <= 1;
    ball1dirDown  <= 1;
	
	ball2vpos <= {0,vmax[11:1]};
    ball2hpos <= {0,hmax[11:1]};;
    ball2dirRight <= 0;
    ball2dirDown  <= 1;
	
	ball3vpos <= {0,vmax[11:1]};
    ball3hpos <= {0,hmax[11:1]};;
    ball3dirRight <= 1;
    ball3dirDown  <= 0;
   
  end
	   
  end
  
  
  always @(posedge SysClkxCI ) begin
   if (ImgRxPixValxMI) begin
     if (startOfFrame) begin
		ball1VertEnable <= 0;
		ball2VertEnable <= 0;
		ball3VertEnable <= 0;
     end else if (startOfLine) begin
	
	   if (vcnt==ball1vpos) begin
	     ball1PatternVert <= 0;
		 ball1VertEnable  <= 1;
	   end
       else if (ball1VertEnable) begin
           ball1PatternVert <= ball1PatternVert + 1'b1;
		   if (ball1PatternVert==patMax) ball1VertEnable <= 1'b0;
       end
	   
	   if (vcnt==ball2vpos) begin
	     ball2PatternVert <= 0;
		 ball2VertEnable  <= 1;
	   end
       else if (ball2VertEnable) begin
           ball2PatternVert <= ball2PatternVert + 1'b1;
		   if (ball2PatternVert==patMax) ball2VertEnable <= 1'b0;
       end
	   
	   if (vcnt==ball3vpos) begin
	     ball3PatternVert <= 0;
		 ball3VertEnable  <= 1;
	   end
       else if (ball3VertEnable) begin
           ball3PatternVert <= ball3PatternVert + 1'b1;
		   if (ball3PatternVert==patMax) ball3VertEnable <= 1'b0;
       end
	   
	   
	 end 
   end
   
  end
  
  always @(posedge SysClkxCI ) begin
   if (ImgRxPixValxMI) begin
	if (startOfLine) begin
		ball1HorEnable<=1'b0;
		ball2HorEnable<=1'b0;
		ball3HorEnable<=1'b0;
	end
	else begin
	  if (hcnt==ball1hpos) begin
	    ball1PatternHor <= 0;
		ball1HorEnable  <= 1;
      end
      else begin
        ball1PatternHor <= ball1PatternHor + 1'b1;
		if (ball1PatternHor==patMax) ball1HorEnable<=1'b0;
      end
	  
	  if (hcnt==ball2hpos) begin
	    ball2PatternHor <= 0;
		ball2HorEnable  <= 1;
      end
      else begin
        ball2PatternHor <= ball2PatternHor + 1'b1;
		if (ball2PatternHor==patMax) ball2HorEnable<=1'b0;
      end
	  
	  if (hcnt==ball3hpos) begin
	    ball3PatternHor <= 0;
		ball3HorEnable  <= 1;
      end
      else begin
        ball3PatternHor <= ball3PatternHor + 1'b1;
		if (ball3PatternHor==patMax) ball3HorEnable<=1'b0;
      end
	  
	end
   end
  end
  
  
  
  
  
  // -----------------------------------------------------
  //            System bus register bank
  // -----------------------------------------------------
  
  // --- system bus read ---
  always @(posedge SysClkxCI ) begin
    if (SysReqxMI & !SysWrEnxMI) begin
      if (regId==0 ) begin
	     cbReadOut <= 32'h40414243;
	  end
	  else if (regId==1 ) begin
	     cbReadOut <= {31'b0000000000000000000000000000000,paramEnable};
	  end
	  else if (regId==2 ) begin
	     cbReadOut <= {29'b00000000000000000000000000000,paramSpeed};
	  end
	  else if (regId==3 ) begin
	     cbReadOut <= {29'b00000000000000000000000000000,paramSize};
	  end
	  else if (regId==4 ) begin
	     cbReadOut <= {30'b000000000000000000000000000000,paramColor};
	  end
	  else if (regId==5 ) begin
	     cbReadOut <= fcnt;
	  end
	  
	  else if (regId==32'h00007fd0 ) begin
	     cbReadOut <= {32'b00000000000000000000000000000001}; // enable message transmitter
	  end
	  
	  
	  else if (regId==32'h00007fd1 ) begin
	     cbReadOut <= msgcnt; // value change triggers message transmitter
	  end
	  
	  else if (regId==32'h00007fd2 ) begin
	     cbReadOut <= msgdata1; // message data1
	  end
	  
	  else if (regId==32'h00007fd3 ) begin
	     cbReadOut <= msgdata2; // message data2
	  end
	  
	  else if (regId==32'h00007fd4 ) begin
	     cbReadOut <= 32'h000000ff; // message data3
	  end
	  
	  else if (regId==32'h00007fd5 ) begin
	     cbReadOut <= 32'h00000001; // message data4
	  end
	 
	  	  
	  else begin
	     cbReadOut <= 0;
	  end
    end
	else begin 
	   cbReadOut <= 0;
	end
  end
  
  // --- system bus write ---
  always @(posedge SysClkxCI or posedge SysRstxRI) begin
  
    if (SysRstxRI) begin
	   paramEnable <= 0;
	   paramSpeed  <= 3'b010;
	   paramSize   <= 3'b110;
	   paramColor  <= 1;
	end
  
    else if (SysReqxMI & SysWrEnxMI) begin
      
	    if (regId==1 ) begin
		  paramEnable <= SysWrDataxDI[0];
		end
		
		if (regId==2 ) begin
		  paramSpeed <= SysWrDataxDI[2:0];
		end
		
		if (regId==3 ) begin
		  paramSize <= SysWrDataxDI[2:0];
		end
		
		if (regId==4 ) begin
		  paramColor <= SysWrDataxDI[0];
		end
	  
    end
  end
  
  always @(posedge SysClkxCI ) begin
    if (AckReg) begin
	   AckReg <= 1'b0;
	end
	else if (SysReqxMI) begin
	   AckReg <= 1'b1;
	end
  end
  
  
  assign SysAckxMO = AckReg;//1'b1;
  assign SysErrxMO = 1'b0;
  assign SysRdDataxDO = cbReadOut;
  
  // --- image output interface ---
  
  assign  ImgTxPixDataxDO = ImgTxPixDataxReg;
  assign  ImgTxPixStatxSO = ImgTxPixStatxReg;
  assign  ImgTxPixValxMO  = ImgTxPixValxReg;
  
  assign  ImgRxPixRdyxMO = 1'b1;
  
  assign CustIoCtrlxMO    = 6'b000000;
  assign CustIoDataOutxDO = 6'b000000;
  
  
  
   px_null32 u_px_null32 (
    .px_cmd_addr_o        ( px_cmd_addr_o        ),
    .px_cmd_bl_o          ( px_cmd_bl_o          ),
    .px_cmd_en_o          ( px_cmd_en_o          ),
    .px_cmd_full_i        ( px_cmd_full_i        ),
    .px_cmd_instr_o       ( px_cmd_instr_o       ),
    .px_rd_data_i         ( px_rd_data_i         ),
    .px_rd_empty_i        ( px_rd_empty_i        ),
    .px_rd_en_o           ( px_rd_en_o           ),
    .px_sys_clk           ( px_sys_clk           ),
    .px_wr_data_o         ( px_wr_data_o         ),
    .px_wr_en_o           ( px_wr_en_o           ),
    .px_wr_full_i         ( px_wr_full_i         )
  );
  

  
endmodule

