//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : px_null32
// Submodules : non
//
// Purpose    : unused px port
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module px_null32 (
    // Xilinx MCB command port
    px_cmd_addr_o,
    px_cmd_bl_o,
    px_cmd_en_o,
    px_cmd_full_i,
    px_cmd_instr_o,

    // Xilinx MCB read port
    px_rd_data_i,
    px_rd_empty_i,
    px_rd_en_o,

    // system signals
    px_sys_clk,

    // Xilinx MCB write port
    px_wr_data_o,
    px_wr_en_o,
    px_wr_full_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // Xilinx MCB command port
  //--------------------------------------------------------
  output [29:0]   px_cmd_addr_o;        // Byte start address for current
                                        // transaction. Addresses must be
                                        // aligned to port size:
                                        // 32-bit ports: Lower two bits must be
                                        //  0s.
  output [5:0]    px_cmd_bl_o;          // Burst length in number of user words
                                        //  for the current transaction. Burst
                                        // length is encoded as 0 to 63,
                                        // representing 1 to 64 user words [for
                                        //  example, b00011 is a burst length 4
                                        //  transaction]. The user word width
                                        // equals the port width [for example,
                                        // a burst length of 3 on a 64-bit port
                                        //  transfers 3 x 64-bit user words =
                                        // 192 bits total].
  output          px_cmd_en_o;          // This active-High signal is the
                                        // write-enable signal for the Command
                                        // FIFO. This signal is covered in more
                                        //  detail in Chapter 4.
  input           px_cmd_full_i;        // This active-High output is the full
                                        // flag for the Command
                                        // FIFO. It indicates the FIFO cannot
                                        // accept any more
                                        // commands and blocks writes to the
                                        // Command FIFO.
  output [2:0]    px_cmd_instr_o;       // Command code for the current
                                        // instruction. Bit 0 represents the
                                        // READ/WRITE select, Bit 1 is Auto
                                        // Precharge enable, and Bit 2
                                        // represents Refresh, which always
                                        // takes priority:
                                        // Write: 3b000
                                        // Read: 3b001
                                        // This signal is covered in more
                                        // detail in Chapter 4.

  //--------------------------------------------------------
  // Xilinx MCB read port
  //--------------------------------------------------------
  input  [31:0]   px_rd_data_i;         // Read Data value returning from
                                        // memory. This signal is driven by the
                                        //  output of the Read Data FIFO into
                                        // FPGA logic. PX_SIZE can be 32, 64,
                                        // or 128 bits, depending on the port
                                        // configuration.
  input           px_rd_empty_i;        // This active-High signal is the empty
                                        //  flag for the Read Data FIFO. It
                                        // indicates there is no valid data in
                                        // the FIFO.
  output          px_rd_en_o;           // This active-High signal is the read
                                        // enable for the Read Data FIFO. Read
                                        // Data is clocked out of the FIFO on
                                        // the rising edge of pX_rd_clk when
                                        // pX_rd_en = 1 and pX_rd_empty = 0.

  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  output          px_sys_clk;           // system clock

  //--------------------------------------------------------
  // Xilinx MCB write port
  //--------------------------------------------------------
  output [31:0]   px_wr_data_o;         // Write Data value to be loaded into
                                        // Write Data FIFO and sent to memory.
                                        // PX_SIZE can be 32, 64, or 128 bits,
                                        // depending on port configuration.
  output          px_wr_en_o;           // This active-High signal is the write
                                        //  enable for the Write Data FIFO. It
                                        // indicates that the value on
                                        // pX_wr_data is valid for
                                        // loading into the FIFO. Data is
                                        // loaded on the rising edge of
                                        // pX_wr_clk when pX_wr_en = 1 and
                                        // pX_wr_full = 0.
  input           px_wr_full_i;         // This active-High signal is the full
                                        // flag for the Write Data FIFO. When
                                        // this signal is high, it prevents
                                        // data from being loade into the FIFO.


  assign px_cmd_addr_o  = 30'd0;
  assign px_cmd_bl_o    = 6'd0;
  assign px_cmd_en_o    = 1'd0;
  assign px_cmd_instr_o = 3'd0;
  assign px_rd_en_o     = 1'd0;
  assign px_wr_data_o   = 32'd0;
  assign px_wr_en_o     = 1'd0;
  assign px_sys_clk     = 1'd0;
endmodule
