//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : CustomModule
// Submodules : cmif2sif
//              siftop
//
// Purpose    : CustomModule Top Level
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module CustomModule (
    // Module Infrastructure Signals
    ImgRstxRI,
    SysClkxCI,
    SysRstxRI,

    // ImageBus Receive Port
    ImgRxPixDataxDI,
    ImgRxPixRdyxMO,
    ImgRxPixStatxSI,
    ImgRxPixValxMI,

    // ImageBus Transmit Port
    ImgTxPixDataxDO,
    ImgTxPixRdyxMI,
    ImgTxPixStatxSO,
    ImgTxPixValxMO,

    // SystemBus Control Interface
    SysAckxMO,
    SysAddrxAI,
    SysErrxMO,
    SysRdDataxDO,
    SysReqxMI,
    SysWrDataxDI,
    SysWrEnxMI,
    
    // Custom Module GPIO
    CustIoCtrlxMO,
    CustIoDataOutxDO,
    CustIoDataInxDI,

    // command
    px_cmd_addr_o,
    px_cmd_bl_o,
    px_cmd_en_o,
    px_cmd_full_i,
    px_cmd_instr_o,

    // read
    px_rd_data_i,
    px_rd_empty_i,
    px_rd_en_o,

    // write
    px_wr_data_o,
    px_wr_en_o,
    px_wr_full_i
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // Module Infrastructure Signals
  //--------------------------------------------------------
  input           ImgRstxRI;            // ImagePipeline Reset
  input           SysClkxCI;            // Custom Module Clock
  input           SysRstxRI;            // Custom Module Reset

  //--------------------------------------------------------
  // ImageBus Receive Port
  //--------------------------------------------------------
  input  [35:0]   ImgRxPixDataxDI;      // packed data
  output          ImgRxPixRdyxMO;       // receive Pixel Ready - ignore if
                                        // cIMG_HANDSHAKE_EN = False
  input  [2:0]    ImgRxPixStatxSI;      // 000 No-Status (NONE)
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-single-Line-Frame
                                        // (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)
  input           ImgRxPixValxMI;       // transaction valid

  //--------------------------------------------------------
  // ImageBus Transmit Port
  //--------------------------------------------------------
  output [35:0]   ImgTxPixDataxDO;      // packed data
  input           ImgTxPixRdyxMI;       // ImageBus Transmit Pixel Ready -
                                        // ignore if cIMG_HANDSHAKE_EN = False
  output [2:0]    ImgTxPixStatxSO;      // 000 No-Status (NONE)
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-single-Line-Frame
                                        // (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)
  output          ImgTxPixValxMO;       // transaction valid

  //--------------------------------------------------------
  // SystemBus Control Interface
  //--------------------------------------------------------
  output          SysAckxMO;            // indicates successful cycle
                                        // termination
  input  [16:0]   SysAddrxAI;           // address input array
  output          SysErrxMO;            // indicates abnormal cycle termination
  output [31:0]   SysRdDataxDO;         // read data
  input           SysReqxMI;            // cycle input indicating a valid bus
                                        // cycle
  input  [31:0]   SysWrDataxDI;         // write data
  input           SysWrEnxMI;           // write enable
  
  //--------------------------------------------------------
  // Custom Module GPIO
  //--------------------------------------------------------
  output [5:0]    CustIoCtrlxMO;        // GPIO Output Control
  output [5:0]    CustIoDataOutxDO;     // GPIO Output Data
  input  [5:0]    CustIoDataInxDI;      // GPIO Input Data (aktiv Low)

  //--------------------------------------------------------
  // command
  //--------------------------------------------------------
  output [29:0]   px_cmd_addr_o;        // Byte start address for current
                                        // transaction. Addresses must be
                                        // aligned to port size:
                                        // 32-bit ports: Lower two bits must be
                                        //  0s.
  output [5:0]    px_cmd_bl_o;          // Burst length in number of user words
                                        //  for the current transaction. Burst
                                        // length is encoded as 0 to 63,
                                        // representing 1 to 64 user words [for
                                        //  example, b00011 is a burst length 4
                                        //  transaction]. The user word width
                                        // equals the port width [for example,
                                        // a burst length of 3 on a 64-bit port
                                        //  transfers 3 x 64-bit user words =
                                        // 192 bits total].
  output          px_cmd_en_o;          // This active-High signal is the
                                        // write-enable signal for the Command
                                        // FIFO. This signal is covered in more
                                        //  detail in Chapter 4.
  input           px_cmd_full_i;        // This active-High output is the full
                                        // flag for the Command FIFO. It
                                        // indicates the FIFO cannot accept any
                                        //  more commands and blocks writes to
                                        // the Command FIFO.
  output [2:0]    px_cmd_instr_o;       // Command code for the current
                                        // instruction. Bit 0 represents the
                                        // READ/WRITE select, Bit 1 is Auto
                                        // Precharge enable, and Bit 2
                                        // represents Refresh, which always
                                        // takes priority:
                                        // Write: 3b000
                                        // Read: 3b001
                                        // This signal is covered in more
                                        // detail in Chapter 4.

  //--------------------------------------------------------
  // read
  //--------------------------------------------------------
  input  [31:0]   px_rd_data_i;         // Read Data value returning from
                                        // memory. This signal is driven by the
                                        //  output of the Read Data FIFO into
                                        // FPGA logic. PX_SIZE can be 32, 64,
                                        // or 128 bits, depending on the port
                                        // configuration.
  input           px_rd_empty_i;        // This active-High signal is the empty
                                        //  flag for the Read Data FIFO. It
                                        // indicates there is no valid data in
                                        // the FIFO.
  output          px_rd_en_o;           // This active-High signal is the read
                                        // enable for the Read Data FIFO. Read
                                        // Data is clocked out of the FIFO on
                                        // the rising edge of pX_rd_clk when
                                        // pX_rd_en = 1 and pX_rd_empty = 0.

  //--------------------------------------------------------
  // write
  //--------------------------------------------------------
  output [31:0]   px_wr_data_o;         // Write Data value to be loaded into
                                        // Write Data FIFO and sent to memory.
                                        // PX_SIZE can be 32, 64, or 128 bits,
                                        // depending on port configuration.
  output          px_wr_en_o;           // This active-High signal is the write
                                        //  enable for the Write Data FIFO. It
                                        // indicates that the value on
                                        // pX_wr_data is valid for loading into
                                        //  the FIFO. Data is loaded on the
                                        // rising edge of pX_wr_clk when
                                        // pX_wr_en = 1 and pX_wr_full = 0.
  input           px_wr_full_i;         // This active-High signal is the full
                                        // flag for the Write Data FIFO. When
                                        // this signal is high, it prevents
                                        // data from being loade into the FIFO.

  //----------------------------------------------------------------------------
  // wire declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  wire            clk;                  // system clock
  wire            sys_rst_n;            // asynchronous system reset, active low
  wire            img_rst_n;            // asynchronous image bus reset, active low

  //--------------------------------------------------------
  // config OCP
  //--------------------------------------------------------
  wire   [16:2]   sif_cfg_maddr_s;      // address
  wire   [1:0]    sif_cfg_mcmd_s;       // command, 0:NOP, 1:write, 2:read
  wire   [31:0]   sif_cfg_mdata_s;      // write data
  wire            sif_cfg_mrespaccept_s;// read response accept
  wire            sif_cfg_scmdaccept_s; // command accept
  wire   [31:0]   sif_cfg_sdata_s;      // read data
  wire            sif_cfg_sresp_s;      // read response

  //--------------------------------------------------------
  // frame data input
  //--------------------------------------------------------
  wire   [11:0]   sif_din_b_s;          // blue
  wire            sif_din_evt_s;        // transaction event, 1cc pulse
  wire            sif_din_fe_s;         // frame end
  wire            sif_din_fs_s;         // frame start
  wire   [11:0]   sif_din_g_s;          // green
  wire            sif_din_le_s;         // line end
  wire            sif_din_ls_s;         // line start
  wire   [11:0]   sif_din_r_s;          // red

  //--------------------------------------------------------
  // frame data output
  //--------------------------------------------------------
  wire   [11:0]   sif_dout_b_s;         // blue
  wire            sif_dout_evt_s;       // transaction event, 1cc pulse
  wire            sif_dout_fe_s;        // frame end
  wire            sif_dout_fs_s;        // frame start
  wire   [11:0]   sif_dout_g_s;         // green
  wire            sif_dout_le_s;        // line end
  wire            sif_dout_ls_s;        // line start
  wire   [11:0]   sif_dout_r_s;         // red


  wire            sif_proc_en = ImgTxPixRdyxMI;
  
  
  assign CustIoCtrlxMO = 6'b0;
  assign CustIoDataOutxDO = 6'b0;

  //--------------------------------------------------------
  // instantiation of module -- CMIF2SIF --
  //--------------------------------------------------------
  cmif2sif u_cmif2sif (
    .clk                          ( clk                          ),
    .sys_rst_n                    ( sys_rst_n                    ),
    .img_rst_n                    ( img_rst_n                    ),
    .ImgRstxRI                    ( ImgRstxRI                    ),
    .SysClkxCI                    ( SysClkxCI                    ),
    .SysRstxRI                    ( SysRstxRI                    ),
    .cfg_maddr_o                  ( sif_cfg_maddr_s              ),
    .cfg_mcmd_o                   ( sif_cfg_mcmd_s               ),
    .cfg_mdata_o                  ( sif_cfg_mdata_s              ),
    .cfg_mrespaccept_o            ( sif_cfg_mrespaccept_s        ),
    .cfg_scmdaccept_i             ( sif_cfg_scmdaccept_s         ),
    .cfg_sdata_i                  ( sif_cfg_sdata_s              ),
    .cfg_sresp_i                  ( sif_cfg_sresp_s              ),
    .ImgRxPixDataxDI              ( ImgRxPixDataxDI              ),
    .ImgRxPixRdyxMO               ( ImgRxPixRdyxMO               ),
    .ImgRxPixStatxSI              ( ImgRxPixStatxSI              ),
    .ImgRxPixValxMI               ( ImgRxPixValxMI               ),
    .ImgTxPixDataxDO              ( ImgTxPixDataxDO              ),
    .ImgTxPixRdyxMI               ( ImgTxPixRdyxMI               ),
    .ImgTxPixStatxSO              ( ImgTxPixStatxSO              ),
    .ImgTxPixValxMO               ( ImgTxPixValxMO               ),
    .SysAckxMO                    ( SysAckxMO                    ),
    .SysAddrxAI                   ( SysAddrxAI                   ),
    .SysErrxMO                    ( SysErrxMO                    ),
    .SysRdDataxDO                 ( SysRdDataxDO                 ),
    .SysReqxMI                    ( SysReqxMI                    ),
    .SysWrDataxDI                 ( SysWrDataxDI                 ),
    .SysWrEnxMI                   ( SysWrEnxMI                   ),
    .din_b_i                      ( sif_dout_b_s                 ),
    .din_evt_i                    ( sif_dout_evt_s               ),
    .din_fe_i                     ( sif_dout_fe_s                ),
    .din_fs_i                     ( sif_dout_fs_s                ),
    .din_g_i                      ( sif_dout_g_s                 ),
    .din_le_i                     ( sif_dout_le_s                ),
    .din_ls_i                     ( sif_dout_ls_s                ),
    .din_r_i                      ( sif_dout_r_s                 ),
    .dout_b_o                     ( sif_din_b_s                  ),
    .dout_evt_o                   ( sif_din_evt_s                ),
    .dout_fe_o                    ( sif_din_fe_s                 ),
    .dout_fs_o                    ( sif_din_fs_s                 ),
    .dout_g_o                     ( sif_din_g_s                  ),
    .dout_le_o                    ( sif_din_le_s                 ),
    .dout_ls_o                    ( sif_din_ls_s                 ),
    .dout_r_o                     ( sif_din_r_s                  )
  );

  //--------------------------------------------------------
  // instantiation of module -- SIFTOP --
  //--------------------------------------------------------
  siftop u_siftop (
    .clk                          ( clk                          ),
    .sys_rst_n                    ( sys_rst_n                    ),
    .img_rst_n                    ( img_rst_n                    ),
    .cfg_maddr_i                  ( sif_cfg_maddr_s              ),
    .cfg_mcmd_i                   ( sif_cfg_mcmd_s               ),
    .cfg_mdata_i                  ( sif_cfg_mdata_s              ),
    .cfg_mrespaccept_i            ( sif_cfg_mrespaccept_s        ),
    .cfg_scmdaccept_o             ( sif_cfg_scmdaccept_s         ),
    .cfg_sdata_o                  ( sif_cfg_sdata_s              ),
    .cfg_sresp_o                  ( sif_cfg_sresp_s              ),
    .din_b_i                      ( sif_din_b_s                  ),
    .din_evt_i                    ( sif_din_evt_s                ),
    .din_fe_i                     ( sif_din_fe_s                 ),
    .din_fs_i                     ( sif_din_fs_s                 ),
    .din_g_i                      ( sif_din_g_s                  ),
    .din_le_i                     ( sif_din_le_s                 ),
    .din_ls_i                     ( sif_din_ls_s                 ),
    .din_r_i                      ( sif_din_r_s                  ),
    .dout_b_o                     ( sif_dout_b_s                 ),
    .dout_evt_o                   ( sif_dout_evt_s               ),
    .dout_fe_o                    ( sif_dout_fe_s                ),
    .dout_fs_o                    ( sif_dout_fs_s                ),
    .dout_g_o                     ( sif_dout_g_s                 ),
    .dout_le_o                    ( sif_dout_le_s                ),
    .dout_ls_o                    ( sif_dout_ls_s                ),
    .dout_r_o                     ( sif_dout_r_s                 ),
    .proc_en_i                    ( sif_proc_en                  ),
    .px_cmd_addr_o                ( px_cmd_addr_o                ),
    .px_cmd_bl_o                  ( px_cmd_bl_o                  ),
    .px_cmd_en_o                  ( px_cmd_en_o                  ),
    .px_cmd_full_i                ( px_cmd_full_i                ),
    .px_cmd_instr_o               ( px_cmd_instr_o               ),
    .px_rd_data_i                 ( px_rd_data_i                 ),
    .px_rd_empty_i                ( px_rd_empty_i                ),
    .px_rd_en_o                   ( px_rd_en_o                   ),
    .px_wr_data_o                 ( px_wr_data_o                 ),
    .px_wr_en_o                   ( px_wr_en_o                   ),
    .px_wr_full_i                 ( px_wr_full_i                 )
  );

endmodule
