//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \ 
//                                   | |          / /    ___ \ \ 
//                                   | |         / /    |__ \ \ \ 
//                                   | |        / /     __ \ \ \ \ 
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : cmif2sif_rstbuf
// Submodules : non
//
// Purpose    : reset buffer
//
// Creator    : iv-tec
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps


module cmif2sif_rstbuf (
    // Module Infrastructure Signals
    ImgRstxRI,
    SysClkxCI,
    SysRstxRI,
    
    // system signals
    sys_clk,
    sys_rst_n,
    img_rst_n    
  );

  //----------------------------------------------------------------------------
  // port declaration
  //----------------------------------------------------------------------------
  //--------------------------------------------------------
  // Module Infrastructure Signals
  //--------------------------------------------------------
  input           ImgRstxRI;            // ImagePipeline Reset
  input           SysClkxCI;            // Custom Module Clock
  input           SysRstxRI;            // Custom Module Reset
  
  //--------------------------------------------------------
  // system signals
  //--------------------------------------------------------
  output          sys_clk;              // system clock
  output          sys_rst_n;            // asynchron low active reset
  output          img_rst_n;            // asynchron low active reset

  


  assign sys_clk = SysClkxCI;
  reg sys_rst_n;
  reg img_rst_n;
  
  always @(posedge SysClkxCI or posedge SysRstxRI) begin
    if (SysRstxRI)  sys_rst_n <= 1'd0;
    else            sys_rst_n <= 1'd1;
  end

  always @(posedge SysClkxCI or posedge ImgRstxRI) begin
    if (ImgRstxRI)  img_rst_n <= 1'd0;
    else            img_rst_n <= 1'd1;
  end

endmodule
