//------------------------------------------------------------------------------
//                                                  __________
//                                    _            / ________ \
//                                   | |          / /    ___ \ \
//                                   | |         / /    |__ \ \ \
//                                   | |        / /     __ \ \ \ \
//           ________     ________   | |____   /_/ __  /  \ \ \ \ \
//          / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
//         | |      | | | |  ____| | | |            \ \____/ / / /
//         | |      | | | | |_____/  | |             \______/ / /
//         | |      | | | |________  | |________         ____/ /
//         |_|      |_|  \_________|  \_________|       |_____/
//
//                                    NEW ELECTRONIC TECHNOLOGY
//
//------------------------------------------------------------------------------
//
// Copyright (C) 2014 NET GmbH All Rights Reserved.
//
// This document, and its contents are the property of NET GmbH.
// NET GmbH retains all rights to: this document, the document contents,
// and the design techniques expressed by the document contents.
// Neither this document, nor any of the information, nor any of the design
// techniques expressed within may be, either in whole or in part: used,
// reproduced, copied, stored, or conveyed to a third party by any means
// without the express written consent of an authorized representative of
// NET GmbH.
//
//------------------------------------------------------------------------------
// Module     : CustomModule
//------------------------------------------------------------------------------
`timescale 1 ns / 1 ps

module CustomModule (
    // Module Infrastructure Signals
    SysClkxCI,                         // Custom Module Clock 150 MHz
    SysRstxRI,                         // Custom Module Reset
    ImgRstxRI,                         // ImageBus Reset

    // SystemBus Control Interface
    SysReqxMI,                         // SystemBus Cycle Request
    SysAckxMO,                         // SystemBus Cycle Acknowledge
    SysErrxMO,                         // SystemBus Cycle Error
    SysWrEnxMI,                        // SystemBus Write Enable
    SysAddrxAI,                        // SystemBus Address
    SysWrDataxDI,                      // SystemBus Write Data
    SysRdDataxDO,                      // SystemBus Read Data

    // Custom Module GPIO
    CustIoCtrlxMO,                     // GPIO Output Control
    CustIoDataOutxDO,                  // GPIO Output Data
    CustIoDataInxDI,                   // GPIO Input Data

    // ImageBus Receive Port
    ImgRxPixValxMI,                    // ImageBus Receive Pixel Valid
    ImgRxPixRdyxMO,                    // ImageBus Receive Pixel Ready
    ImgRxPixStatxSI,                   // ImageBus Receive Pixel Status
    ImgRxPixDataxDI,                   // ImageBus Receive Pixel Data

    // ImageBus Transmit Port
    ImgTxPixValxMO,                    // ImageBus Transmit Pixel Valid
    ImgTxPixRdyxMI,                    // ImageBus Transmit Pixel Ready
    ImgTxPixStatxSO,                   // ImageBus Transmit Pixel Status
    ImgTxPixDataxDO,                   // ImageBus Transmit Pixel Data

    // Custom Module MCB Command Port
    px_cmd_en_o,                       // MCB Command Valid
    px_cmd_full_i,                     // MCB Command Buffer Full
    px_cmd_instr_o,                    // MCB Command Instruction
    px_cmd_bl_o,                       // MCB Command Burst Length
    px_cmd_addr_o,                     // MCB Command Address

    // Custom Module MCB Read Port
    px_rd_en_o,                        // MCB Read Data Valid
    px_rd_empty_i,                     // MCB Read Data Buffer Empty
    px_rd_data_i,                      // MCB Read Data Bus

    // Custom Module MCB Write Port
    px_wr_en_o,                        // MCB Write Data Valid
    px_wr_full_i,                      // MCB Write Data Buffer Full
    px_wr_data_o                       // MCB Write Data Bus
  );

  //----------------------------------------------------------------------------
  // Port Declaration
  //----------------------------------------------------------------------------

  //--------------------------------------------------------
  // Module Infrastructure Signals
  //--------------------------------------------------------
  input           SysClkxCI;            // Custom Module Clock
  input           SysRstxRI;            // Custom Module Reset
  input           ImgRstxRI;            // ImagePipeline Reset

  //--------------------------------------------------------
  // SystemBus Control Interface
  //--------------------------------------------------------
  input           SysReqxMI;            // cycle input indicating a valid bus cycle
  output          SysAckxMO;            // indicates successful cycle termination
  output          SysErrxMO;            // indicates abnormal cycle termination
  input           SysWrEnxMI;           // write enable
  input  [16:0]   SysAddrxAI;           // address input array
  input  [31:0]   SysWrDataxDI;         // write data
  output [31:0]   SysRdDataxDO;         // read data

  //--------------------------------------------------------
  // Custom Module GPIO
  //--------------------------------------------------------
  output [5:0]    CustIoCtrlxMO;        // GPIO Output Control for CustIoDataOutxDO
  output [5:0]    CustIoDataOutxDO;     // GPIO Output Data
  input  [5:0]    CustIoDataInxDI;      // GPIO Input Data

  //--------------------------------------------------------
  // ImageBus Receive Port
  //--------------------------------------------------------
  input  [35:0]   ImgRxPixDataxDI;      // packed data

  output          ImgRxPixRdyxMO;       // receive Pixel Ready - ignore if
                                        // cIMG_HANDSHAKE_EN = False

  input  [2:0]    ImgRxPixStatxSI;      // 000 No-Status (NONE)
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-single-Line-Frame (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)

  input           ImgRxPixValxMI;       // transaction valid

  //--------------------------------------------------------
  // ImageBus Transmit Port
  //--------------------------------------------------------
  output [35:0]   ImgTxPixDataxDO;      // packed data

  input           ImgTxPixRdyxMI;       // ImageBus Transmit Pixel Ready -
                                        // ignore if cIMG_HANDSHAKE_EN = False

  output [2:0]    ImgTxPixStatxSO;      // 000 No-Status (NONE)
                                        // 001 Start-of-Frame (SOF)
                                        // 010 Start-of-single-Line-Frame (SOSLF)
                                        // 011 Start-of-Data (SOD)
                                        // 100 Start-of-Line (SOL)
                                        // 101 Start-of-Last-Line (SOLL)
                                        // 110 End-of-Line/Data (EOL)
                                        // 111 Data-Error (ERR)

  output          ImgTxPixValxMO;       // transaction valid

  //--------------------------------------------------------
  // Custom Module MCB Command Port
  //--------------------------------------------------------
  output          px_cmd_en_o;          // This active-High signal is the
                                        // write-enable signal for the Command FIFO.

  input           px_cmd_full_i;        // This active-High output is the full
                                        // flag for the Command FIFO. It
                                        // indicates the FIFO cannot accept any
                                        // more commands and blocks writes to
                                        // the Command FIFO.

  output [2:0]    px_cmd_instr_o;       // Command code for the current
                                        // instruction. Bit 0 represents the
                                        // READ/WRITE select, Bit 1 is Auto
                                        // Precharge enable, and Bit 2
                                        // represents Refresh, which always
                                        // takes priority:
                                        // Write: 3b000
                                        // Read: 3b001

  output [5:0]    px_cmd_bl_o;          // Burst length in number of user words
                                        // for the current transaction. Burst
                                        // length is encoded as 0 to 63,
                                        // representing 1 to 64 user words [for
                                        // example, b00011 is a burst length 4
                                        // transaction]. The user word width
                                        // equals the port width [for example,
                                        // a burst length of 3 on a 64-bit port
                                        // transfers 3 x 64-bit user words =
                                        // 192 bits total].

  output [29:0]   px_cmd_addr_o;        // Byte start address for current
                                        // transaction. Addresses must be
                                        // aligned to port size:
                                        // 32-bit ports: Lower two bits must be 0s.

  //--------------------------------------------------------
  // Custom Module MCB Read Port
  //--------------------------------------------------------
  output          px_rd_en_o;           // This active-High signal is the read
                                        // enable for the Read Data FIFO. Read
                                        // Data is clocked out of the FIFO on
                                        // the rising edge of pX_rd_clk when
                                        // pX_rd_en = 1 and pX_rd_empty = 0.

  input           px_rd_empty_i;        // This active-High signal is the empty
                                        // flag for the Read Data FIFO. It
                                        // indicates there is no valid data in
                                        // the FIFO.

  input  [31:0]   px_rd_data_i;         // Read Data value returning from
                                        // memory. This signal is driven by the
                                        // output of the Read Data FIFO into
                                        // FPGA logic. PX_SIZE can be 32, 64,
                                        // or 128 bits, depending on the port
                                        // configuration.

  //--------------------------------------------------------
  // Custom Module MCB Write Port
  //--------------------------------------------------------
  output          px_wr_en_o;           // This active-High signal is the write
                                        // enable for the Write Data FIFO. It
                                        // indicates that the value on
                                        // pX_wr_data is valid for loading into
                                        // the FIFO. Data is loaded on the
                                        // rising edge of pX_wr_clk when
                                        // pX_wr_en = 1 and pX_wr_full = 0.

  input           px_wr_full_i;         // This active-High signal is the full
                                        // flag for the Write Data FIFO. When
                                        // this signal is high, it prevents
                                        // data from being loade into the FIFO.

  output [31:0]   px_wr_data_o;         // Write Data value to be loaded into
                                        // Write Data FIFO and sent to memory.
                                        // PX_SIZE can be 32, 64, or 128 bits,
                                        // depending on port configuration.


  // Connect ImageBus input port to output port
  assign ImgTxPixValxMO  = ImgRxPixValxMI;
  assign ImgTxPixStatxSO = ImgRxPixStatxSI;
  assign ImgTxPixDataxDO = ImgRxPixDataxDI;
  assign ImgRxPixRdyxMO  = ImgTxPixRdyxMI;

  // Define unused output signals
  assign SysAckxMO    = SysReqxMI;
  assign SysErrxMO    = 1'b0;
  assign SysRdDataxDO = 32'b0;

  assign CustIoCtrlxMO    = 6'b0;
  assign CustIoDataOutxDO = 6'b0;

  assign px_cmd_en_o    = 1'b0;
  assign px_cmd_instr_o = 3'b0;
  assign px_cmd_bl_o    = 6'b0;
  assign px_cmd_addr_o  = 30'b0;

  assign px_rd_en_o     = 1'b0;

  assign px_wr_en_o     = 1'b0;
  assign px_wr_data_o   = 32'b0;

endmodule
