-- ------------------------------------------------------------------------------
-- File Name         :  $RCSfile:  $
-- Author            :  $Author:  $
-- Revision          :  $Revision:  $
-- Last Modification :  $Date:  $
-- ------------------------------------------------------------------------------
--                                                 __________
--                                   _            / ________ \ 
--                                  | |          / /    ___ \ \ 
--                                  | |         / /    |__ \ \ \ 
--                                  | |        / /     __ \ \ \ \ 
--          ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
--         / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
--        | |      | | | |  ____| | | |            \ \____/ / / /
--        | |      | | | | |_____/  | |             \______/ / /
--        | |      | | | |________  | |________         ____/ /
--        |_|      |_|  \_________|  \_________|       |_____/
--
--                                   NEW ELECTRONIC TECHNOLOGY
--
-- ------------------------------------------------------------------------------
--
-- Copyright (C) 2014 NET GmbH All Rights Reserved.
--
-- This document, and its contents are the property of NET GmbH.
-- NET GmbH retains all rights to: this document, the document contents,
-- and the design techniques expressed by the document contents.
-- Neither this document, nor any of the information, nor any of the design
-- techniques expressed within may be, either in whole or in part: used,
-- reproduced, copied, stored, or conveyed to a third party by any means
-- without the express written consent of an authorized representative of
-- NET GmbH.
--
-- ------------------------------------------------------------------------------
-- Module     : CustomModule
-- Submodules : diffpic_cm_gpio_proc_bw
--              diffpic_cm_gpio_regif
--
-- Purpose    : difference picture
--
-- Interface Description:
--    ----------------------------------------------------------------
--    -- Module Infrastructure Signals
--    ----------------------------------------------------------------
--    ImgRstxRI:             ImagePipeline Reset
--    SysClkxCI:             Custom Module Clock
--    SysRstxRI:             Custom Module Reset
--    ----------------------------------------------------------------
--    -- ImageBus Receive Port
--    ----------------------------------------------------------------
--    ImgRxPixDataxDI:       packed data
--    ImgRxPixRdyxMO:        receive Pixel Ready - ignore if
--                           cIMG_HANDSHAKE_EN = False
--    ImgRxPixStatxSI:       000 No-Status (NONE)
--                           001 Start-of-Frame (SOF)
--                           010 Start-of-single-Line-Frame (SOSLF)
--                           011 Start-of-Data (SOD)
--                           100 Start-of-Line (SOL)
--                           101 Start-of-Last-Line (SOLL)
--                           110 End-of-Line/Data (EOL)
--                           111 Data-Error (ERR)
--    ImgRxPixValxMI:        transaction valid
--    ----------------------------------------------------------------
--    -- ImageBus Transmit Port
--    ----------------------------------------------------------------
--    ImgTxPixDataxDO:       packed data
--    ImgTxPixRdyxMI:        ImageBus Transmit Pixel Ready -
--                           ignore if cIMG_HANDSHAKE_EN = False
--    ImgTxPixStatxSO:       000 No-Status (NONE)
--                           001 Start-of-Frame (SOF)
--                           010 Start-of-single-Line-Frame (SOSLF)
--                           011 Start-of-Data (SOD)
--                           100 Start-of-Line (SOL)
--                           101 Start-of-Last-Line (SOLL)
--                           110 End-of-Line/Data (EOL)
--                           111 Data-Error (ERR)
--    ImgTxPixValxMO:        transaction valid
--    ----------------------------------------------------------------
--    -- SystemBus Control Interface
--    ----------------------------------------------------------------
--    SysAckxMO:             indicates successful cycle
--                           termination
--    SysAddrxAI:            address input array
--    SysErrxMO:             indicates abnormal cycle termination
--    SysRdDataxDO:          read data
--    SysReqxMI:             cycle input indicating a valid bus cycle
--    SysWrDataxDI:          write data
--    SysWrEnxMI:            write enable
--    ----------------------------------------------------------------
--    -- Custom Module GPIO
--    ----------------------------------------------------------------
--    CustIoCtrlxMO:         GPIO Output Control
--                           CustIoCtrlxMO(0)='1'                 -> CustIoDataOutxDO(0) enable 
--                       ... CustIoCtrlxMO(NUM_CM_GPIO_OUT-1)='1' -> CustIoDataOutxDO(NUM_CM_GPIO_OUT-1) enable
--    CustIoDataOutxDO:      GPIO Output Data
--    CustIoDataInxDI:       GPIO Input Data (active Low)
--    ----------------------------------------------------------------
--    -- MCP command
--    ----------------------------------------------------------------
--    px_cmd_addr_o:         Byte start address for current
--                           transaction. Addresses must be
--                           aligned to port size:
--                           32-bit ports: Lower two bits must be
--                           0s.
--    px_cmd_bl_o:           Burst length in number of user words
--                           for the current transaction. Burst
--                           length is encoded as 0 to 63,
--                           representing 1 to 64 user words [for
--                           example, b00011 is a burst length 4
--                           transaction]. The user word width
--                           equals the port width [for example,
--                           a burst length of 3 on a 64-bit port
--                           transfers 3 x 64-bit user words =
--                           192 bits total].
--    px_cmd_en_o:           This active-High signal is the
--                           write-enable signal for the Command
--                           FIFO. This signal is covered in more
--                           detail in Chapter 4.
--    px_cmd_full_i:         This active-High output is the full
--                           flag for the Command FIFO. It
--                           indicates the FIFO cannot accept any
--                           more commands and blocks writes to
--                           the Command FIFO.
--    px_cmd_instr_o:        Command code for the current
--                           instruction. Bit 0 represents the
--                           READ/WRITE select, Bit 1 is Auto
--                           Precharge enable, and Bit 2
--                           represents Refresh, which always
--                           takes priority:
--                           Write: 3b000
--                           Read: 3b001
--                           This signal is covered in more
--                           detail in Chapter 4.
--    ----------------------------------------------------------------
--    -- MCB read          
--    ----------------------------------------------------------------
--    px_rd_data_i:          Read Data value returning from
--                           memory. This signal is driven by the
--                           output of the Read Data FIFO into
--                           FPGA logic. PX_SIZE can be 32, 64,
--                           or 128 bits, depending on the port
--                           configuration.
--    px_rd_empty_i:         This active-High signal is the empty
--                           flag for the Read Data FIFO. It
--                           indicates there is no valid data in
--                           the FIFO.
--    px_rd_en_o:            This active-High signal is the read
--                           enable for the Read Data FIFO. Read
--                           Data is clocked out of the FIFO on
--                           the rising edge of pX_rd_clk when
--                           pX_rd_en = 1 and pX_rd_empty = 0.
--    ----------------------------------------------------------------
--    -- MCP write          
--    ----------------------------------------------------------------
--    px_wr_data_o:          Write Data value to be loaded into
--                           Write Data FIFO and sent to memory.
--                           PX_SIZE can be 32, 64, or 128 bits,
--                           depending on port configuration.
--    px_wr_en_o:            This active-High signal is the write
--                           enable for the Write Data FIFO. It
--                           indicates that the value on
--                           pX_wr_data is valid for loading into
--                           the FIFO. Data is loaded on the
--                           rising edge of pX_wr_clk when
--                           pX_wr_en = 1 and pX_wr_full = 0.
--    px_wr_full_i:          This active-High signal is the full
--                           flag for the Write Data FIFO. When
--                           this signal is high, it prevents
--                           data from being loade into the FIFO.
--
--
-- Creator    : a.kramer
-- ------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library work;
use work.diffpic_cm_gpio_pkg.all;


entity CustomModule is
	
  port(
    -- Module Infrastructure Signals
  	ImgRstxRI:					  in std_logic;
  	SysClkxCI:					  in std_logic;
  	SysRstxRI:            in std_logic;
  	-- ImageBus Receive Port
  	ImgRxPixDataxDI:	    in std_logic_vector(35 downto 0);
  	ImgRxPixRdyxMO:		    out std_logic;
  	ImgRxPixStatxSI:	    in std_logic_vector(2 downto 0);
  	ImgRxPixValxMI:		    in std_logic;
  	-- ImageBus Transmit Port
  	ImgTxPixDataxDO:      out std_logic_vector(35 downto 0);
    ImgTxPixRdyxMI:       in std_logic;
    ImgTxPixStatxSO:      out std_logic_vector(2 downto 0);
    ImgTxPixValxMO:       out std_logic;
    -- SystemBus Control Interface
  	SysAckxMO:					  out std_logic;
  	SysAddrxAI:					  in std_logic_vector(16 downto 0);
  	SysErrxMO:					  out std_logic;
  	SysRdDataxDO:         out std_logic_vector(31 downto 0);
    SysReqxMI:            in std_logic;
    SysWrDataxDI:         in std_logic_vector(31 downto 0);
    SysWrEnxMI:           in std_logic;
    -- Custom Module GPIO
    CustIoCtrlxMO:        out std_logic_vector(NUM_CM_GPIO_OUT-1 downto 0);
    CustIoDataOutxDO:     out std_logic_vector(NUM_CM_GPIO_OUT-1 downto 0);
    CustIoDataInxDI:      in  std_logic_vector(NUM_CM_GPIO_IN-1 downto 0);
    -- MCP command
    px_cmd_addr_o:        out std_logic_vector(29 downto 0);
    px_cmd_bl_o:          out std_logic_vector(5 downto 0);
    px_cmd_en_o:          out std_logic;
    px_cmd_full_i:        in std_logic;
    px_cmd_instr_o:       out std_logic_vector(2 downto 0);
    -- MCB read
    px_rd_data_i:         in std_logic_vector(31 downto 0);
    px_rd_empty_i:        in std_logic;
    px_rd_en_o:           out std_logic;
    -- MCP write
    px_wr_data_o:         out std_logic_vector(31 downto 0);
    px_wr_en_o:           out std_logic;
    px_wr_full_i:         in std_logic
  	);	
end CustomModule;


architecture diffpic_cm_gpio_a of CustomModule is

  component diffpic_cm_gpio_proc_bw_e
    port(SysClkxCI            : in  std_logic;
         SysRstxRI            : in  std_logic;
         ImgRxPixDataxDI      : in  std_logic_vector(35 downto 0);
         ImgRxPixRdyxMO       : out std_logic;
         ImgRxPixStatxSI      : in  std_logic_vector(2 downto 0);
         ImgRxPixValxMI       : in  std_logic;
         ImgTxPixDataxDO      : out std_logic_vector(35 downto 0);
         ImgTxPixRdyxMI       : in  std_logic;
         ImgTxPixStatxSO      : out std_logic_vector(2 downto 0);
         ImgTxPixValxMO       : out std_logic;
         CustIoCtrlxMO        : out std_logic_vector(NUM_CM_GPIO_OUT-1 downto 0);
         CustIoDataOutxDO     : out std_logic_vector(NUM_CM_GPIO_OUT-1 downto 0);
         CustIoDataInxDI      : in  std_logic_vector(NUM_CM_GPIO_IN-1 downto 0); 
         px_cmd_addr_o        : out std_logic_vector(29 downto 0);
         px_cmd_bl_o          : out std_logic_vector(5 downto 0);
         px_cmd_en_o          : out std_logic;
         px_cmd_full_i        : in  std_logic;
         px_cmd_instr_o       : out std_logic_vector(2 downto 0);
         px_rd_data_i         : in  std_logic_vector(31 downto 0);
         px_rd_empty_i        : in  std_logic;
         px_rd_en_o           : out std_logic;
         px_wr_data_o         : out std_logic_vector(31 downto 0);
         px_wr_en_o           : out std_logic;
         px_wr_full_i         : in  std_logic;
         param_mem_ptr_i      : in  std_logic_vector(29 downto 0);
         param_mem_size_i     : in  std_logic_vector(29 downto 0);
         param_mem_rdy_i      : in  std_logic;
         param_diffpic_en_i   : in  std_logic;
         param_gray_offs_i    : in  std_logic_vector(7 downto 0);
         param_motion_thres_i : in  std_logic_vector(31 downto 0));
  end component diffpic_cm_gpio_proc_bw_e;
  
  component diffpic_cm_gpio_regif_e
    port(SysClkxCI            : in  std_logic;
         SysRstxRI            : in  std_logic;
         SysAckxMO            : out std_logic;
         SysAddrxAI           : in  std_logic_vector(16 downto 0);
         SysErrxMO            : out std_logic;
         SysRdDataxDO         : out std_logic_vector(31 downto 0);
         SysReqxMI            : in  std_logic;
         SysWrDataxDI         : in  std_logic_vector(31 downto 0);
         SysWrEnxMI           : in  std_logic;
         param_mem_ptr_o      : out std_logic_vector(29 downto 0);
         param_mem_size_o     : out std_logic_vector(29 downto 0);
         param_mem_rdy_o      : out std_logic;
         param_diffpic_en_o   : out std_logic;
         param_gray_offs_o    : out std_logic_vector(7 downto 0);
         param_motion_thres_o : out std_logic_vector(31 downto 0));
  end component diffpic_cm_gpio_regif_e;
 
  signal param_mem_ptr_s       : std_logic_vector(29 downto 0);
  signal param_mem_size_s      : std_logic_vector(29 downto 0);
  signal param_mem_rdy_s       : std_logic;
  signal param_diffpic_en_s    : std_logic;
  signal param_gray_offs_s     : std_logic_vector(7 downto 0);
  signal param_motion_thres_s  : std_logic_vector(31 downto 0);
 
begin
  

  dp_proc: component diffpic_cm_gpio_proc_bw_e
    port map(SysClkxCI            => SysClkxCI,
             SysRstxRI            => SysRstxRI,
             ImgRxPixDataxDI      => ImgRxPixDataxDI,
             ImgRxPixRdyxMO       => ImgRxPixRdyxMO,
             ImgRxPixStatxSI      => ImgRxPixStatxSI,
             ImgRxPixValxMI       => ImgRxPixValxMI,
             ImgTxPixDataxDO      => ImgTxPixDataxDO,
             ImgTxPixRdyxMI       => ImgTxPixRdyxMI,
             ImgTxPixStatxSO      => ImgTxPixStatxSO,
             ImgTxPixValxMO       => ImgTxPixValxMO,
             CustIoCtrlxMO        => CustIoCtrlxMO,
             CustIoDataOutxDO     => CustIoDataOutxDO,
             CustIoDataInxDI      => CustIoDataInxDI,
             px_cmd_addr_o        => px_cmd_addr_o,
             px_cmd_bl_o          => px_cmd_bl_o,
             px_cmd_en_o          => px_cmd_en_o,
             px_cmd_full_i        => px_cmd_full_i,
             px_cmd_instr_o       => px_cmd_instr_o,
             px_rd_data_i         => px_rd_data_i,
             px_rd_empty_i        => px_rd_empty_i,
             px_rd_en_o           => px_rd_en_o,
             px_wr_data_o         => px_wr_data_o,
             px_wr_en_o           => px_wr_en_o,
             px_wr_full_i         => px_wr_full_i,
             param_mem_ptr_i      => param_mem_ptr_s,
             param_mem_size_i     => param_mem_size_s,
             param_mem_rdy_i      => param_mem_rdy_s,
             param_diffpic_en_i   => param_diffpic_en_s,
             param_gray_offs_i    => param_gray_offs_s,
             param_motion_thres_i => param_motion_thres_s);
             
             
  dp_regif: component diffpic_cm_gpio_regif_e
    port map(SysClkxCI            => SysClkxCI,
             SysRstxRI            => SysRstxRI,
             SysAckxMO            => SysAckxMO,
             SysAddrxAI           => SysAddrxAI,
             SysErrxMO            => SysErrxMO,
             SysRdDataxDO         => SysRdDataxDO,
             SysReqxMI            => SysReqxMI,
             SysWrDataxDI         => SysWrDataxDI,
             SysWrEnxMI           => SysWrEnxMI,
             param_mem_ptr_o      => param_mem_ptr_s,
             param_mem_size_o     => param_mem_size_s,
             param_mem_rdy_o      => param_mem_rdy_s,
             param_diffpic_en_o   => param_diffpic_en_s,
             param_gray_offs_o    => param_gray_offs_s,
             param_motion_thres_o => param_motion_thres_s);

	
end diffpic_cm_gpio_a;