-- ------------------------------------------------------------------------------
-- File Name         :  $RCSfile:  $
-- Author            :  $Author:  $
-- Revision          :  $Revision:  $
-- Last Modification :  $Date:  $
-- ------------------------------------------------------------------------------
--                                                 __________
--                                   _            / ________ \ 
--                                  | |          / /    ___ \ \ 
--                                  | |         / /    |__ \ \ \ 
--                                  | |        / /     __ \ \ \ \ 
--          ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
--         / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
--        | |      | | | |  ____| | | |            \ \____/ / / /
--        | |      | | | | |_____/  | |             \______/ / /
--        | |      | | | |________  | |________         ____/ /
--        |_|      |_|  \_________|  \_________|       |_____/
--
--                                   NEW ELECTRONIC TECHNOLOGY
--
-- ------------------------------------------------------------------------------
--
-- Copyright (C) 2014 NET GmbH All Rights Reserved.
--
-- This document, and its contents are the property of NET GmbH.
-- NET GmbH retains all rights to: this document, the document contents,
-- and the design techniques expressed by the document contents.
-- Neither this document, nor any of the information, nor any of the design
-- techniques expressed within may be, either in whole or in part: used,
-- reproduced, copied, stored, or conveyed to a third party by any means
-- without the express written consent of an authorized representative of
-- NET GmbH.
--
-- ------------------------------------------------------------------------------
-- Module     : diffpic_cm_gpio_regif
-- Submodules : non
--
-- Purpose    :
--
-- Creator    : a.kramer
-- ------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library work;
use work.diffpic_cm_gpio_pkg.all;


entity diffpic_cm_gpio_regif_e is
	
  port(
    -- Module Infrastructure Signals
    SysClkxCI:            in std_logic;
    SysRstxRI:            in std_logic;
    -- SystemBus Control Interface
  	SysAckxMO:					  out std_logic;
  	SysAddrxAI:					  in std_logic_vector(16 downto 0);
  	SysErrxMO:					  out std_logic;
  	SysRdDataxDO:         out std_logic_vector(31 downto 0);
    SysReqxMI:            in std_logic;
    SysWrDataxDI:         in std_logic_vector(31 downto 0);
    SysWrEnxMI:           in std_logic;
    -- Parameter
    param_mem_ptr_o:      out std_logic_vector(29 downto 0);
    param_mem_size_o:     out std_logic_vector(29 downto 0);
    param_mem_rdy_o:      out std_logic;
    param_diffpic_en_o:   out std_logic;
    param_gray_offs_o:    out std_logic_vector(7 downto 0);
    param_motion_thres_o: out std_logic_vector(31 downto 0)
    );	
end diffpic_cm_gpio_regif_e;


architecture diffpic_cm_gpio_regif_a of diffpic_cm_gpio_regif_e is

  signal ad_diffpic_id_sel:     std_logic;
  signal ad_mem_size_sel:       std_logic;
  signal ad_mem_ptr_sel:        std_logic;
  signal ad_mem_rdy_sel:        std_logic;
  signal ad_diffpic_conf_sel:   std_logic;
  signal ad_gray_offs_sel:      std_logic;
  signal ad_motion_thres_sel:   std_logic;

  signal ad_diffpic_id_evt:     std_logic;
  signal ad_mem_ptr_evt:        std_logic;
  signal ad_mem_rdy_evt:        std_logic;
  signal ad_diffpic_conf_evt:   std_logic;
  signal ad_gray_offs_evt:      std_logic;
  signal ad_motion_thres_evt:   std_logic;
 
  signal rdg_rddata:            std_logic_vector (31 downto 0);
   
  signal rdg_diffpic_id:        std_logic_vector (31 downto 0);
  signal rdg_mem_size:          std_logic_vector (31 downto 0);
  signal rdg_mem_ptr:           std_logic_vector (31 downto 0);
  signal rdg_mem_rdy:           std_logic_vector (31 downto 0);
  signal rdg_diffpic_conf:      std_logic_vector (31 downto 0);
  signal rdg_gray_offs:         std_logic_vector (31 downto 0);
  signal rdg_motion_thres:      std_logic_vector (31 downto 0);
 
  signal param_mem_ptr_s:       std_logic_vector (29 downto 0);
  signal param_mem_rdy_s:       std_logic;
  signal param_diffpic_en_s:    std_logic;
  signal param_gray_offs_s:     std_logic_vector (7 downto 0);
  signal param_motion_thres_s:  std_logic_vector (31 downto 0);
  
  signal cg_write_evt:          std_logic;
  signal cg_read_evt:           std_logic;
  signal cg_read_ack:           std_logic;
   
begin
  ------------------------------------------------------------
  --address select
  ------------------------------------------------------------
  
  ad_diffpic_id_sel <= '1'    when SysAddrxAI=('0' & x"0000") else '0'; 
  ad_diffpic_conf_sel <= '1'  when SysAddrxAI=('0' & x"0004") else '0';
  ad_gray_offs_sel <= '1'     when SysAddrxAI=('0' & x"0008") else '0';
  ad_motion_thres_sel <= '1'  when SysAddrxAI=('0' & x"000c") else '0';  
  ad_mem_ptr_sel <= '1'       when SysAddrxAI=('1' & x"fe00") else '0';
  ad_mem_rdy_sel <= '1'       when SysAddrxAI=('1' & x"fe04") else '0';
  ad_mem_size_sel <= '1'      when SysAddrxAI=('1' & x"ff00") else '0'; 

  
  ------------------------------------------------------------
  --address write-event
  ------------------------------------------------------------
  
  ad_mem_ptr_evt  <=  ad_mem_ptr_sel             and cg_write_evt;
  ad_mem_rdy_evt  <=  ad_mem_rdy_sel             and cg_write_evt;
  ad_diffpic_conf_evt <=  ad_diffpic_conf_sel    and cg_write_evt;
  ad_gray_offs_evt <=  ad_gray_offs_sel          and cg_write_evt;
  ad_motion_thres_evt <=  ad_motion_thres_sel    and cg_write_evt;


  ------------------------------------------------------------
  --write data
  ------------------------------------------------------------ 
                                 
  init_wr: process (SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      param_mem_ptr_s  <= "00" & x"0000000";
      param_mem_rdy_s  <= '0';
      param_diffpic_en_s  <= '1';
      param_gray_offs_s   <=  x"80";
      param_motion_thres_s   <=  x"00000000";
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (ad_mem_ptr_evt='1') then
          param_mem_ptr_s  <= SysWrDataxDI(29 downto 0);
      end if;
      if (ad_mem_rdy_evt='1') then
          param_mem_rdy_s  <= SysWrDataxDI(0);
      end if;
      if (ad_diffpic_conf_evt='1') then
          param_diffpic_en_s  <= SysWrDataxDI(0);
      end if;
      if (ad_gray_offs_evt='1') then
          param_gray_offs_s  <= SysWrDataxDI(7 downto 0);
      end if;
      if (ad_motion_thres_evt='1') then
          param_motion_thres_s  <= SysWrDataxDI(31 downto 0);
      end if;
    end if;
  end process;
  
  param_mem_ptr_o  <=  param_mem_ptr_s;
  param_mem_size_o <=  MEM_SIZE;
  param_mem_rdy_o  <=  param_mem_rdy_s;
  param_diffpic_en_o <= param_diffpic_en_s;
  param_gray_offs_o <= param_gray_offs_s;
  param_motion_thres_o <= param_motion_thres_s;
  
  
  ------------------------------------------------------------
  --read data generation
  ------------------------------------------------------------
  
  rdg_rddata <= rdg_diffpic_id        when ad_diffpic_id_sel='1' else
                rdg_mem_size          when ad_mem_size_sel='1' else
                rdg_mem_ptr           when ad_mem_ptr_sel='1' else
                rdg_mem_rdy           when ad_mem_rdy_sel='1' else
                rdg_diffpic_conf      when ad_diffpic_conf_sel='1' else
                rdg_gray_offs         when ad_gray_offs_sel='1' else
                rdg_motion_thres      when ad_motion_thres_sel='1' else
                x"00000000";
              
  rdg_diffpic_id <= DEV_ID;
  rdg_mem_size <= "00" & MEM_SIZE;
  rdg_mem_ptr <= "00" & param_mem_ptr_s;
  rdg_mem_rdy <= x"0000000" & "000" & param_mem_rdy_s;
  rdg_diffpic_conf <= x"0000000" & "000" & param_diffpic_en_s;
  rdg_gray_offs <= x"000000" & param_gray_offs_s;
  rdg_motion_thres <= param_motion_thres_s;
  
  
  
  ------------------------------------------------------------
  --command generation
  ------------------------------------------------------------  
                              
  cg_write_evt <= SysReqxMI and SysWrEnxMI;
  cg_read_evt <= SysReqxMI and not SysWrEnxMI and not cg_read_ack;
  
  init_rd: process (SysClkxCI)
  begin
    if (SysClkxCI'event and SysClkxCI='1') then
      if (cg_read_evt='1') then
        cg_read_ack <= '1';
        SysRdDataxDO  <= rdg_rddata;
      else
        cg_read_ack <= '0';
      end if;
    end if;
  end process;
  
  
  SysAckxMO  <= '1' when cg_write_evt='1' or cg_read_ack='1' else
                '0';
  
  SysErrxMO <= '0';
  
	
end diffpic_cm_gpio_regif_a;