-- ------------------------------------------------------------------------------
-- File Name         :  $RCSfile:  $
-- Author            :  $Author:  $
-- Revision          :  $Revision:  $
-- Last Modification :  $Date:  $
-- ------------------------------------------------------------------------------
--                                                 __________
--                                   _            / ________ \ 
--                                  | |          / /    ___ \ \ 
--                                  | |         / /    |__ \ \ \ 
--                                  | |        / /     __ \ \ \ \ 
--          ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
--         / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
--        | |      | | | |  ____| | | |            \ \____/ / / /
--        | |      | | | | |_____/  | |             \______/ / /
--        | |      | | | |________  | |________         ____/ /
--        |_|      |_|  \_________|  \_________|       |_____/
--
--                                   NEW ELECTRONIC TECHNOLOGY
--
-- ------------------------------------------------------------------------------
--
-- Copyright (C) 2016 NET GmbH All Rights Reserved.
--
-- This document, and its contents are the property of NET GmbH.
-- NET GmbH retains all rights to: this document, the document contents,
-- and the design techniques expressed by the document contents.
-- Neither this document, nor any of the information, nor any of the design
-- techniques expressed within may be, either in whole or in part: used,
-- reproduced, copied, stored, or conveyed to a third party by any means
-- without the express written consent of an authorized representative of
-- NET GmbH.
--
-- ------------------------------------------------------------------------------
-- Module     : scale_d_cm_pkg
-- Submodules : non
--
-- Purpose    :
--
-- Creator    : a.kramer
-- ------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;


package scale_d_cm_pkg is

  constant NUM_CM_GPIO_IN:     integer := 6;                                                    -- number of gpio-inputs
  constant NUM_CM_GPIO_OUT:    integer := 6;                                                    -- number of gpio-outputs
  constant CM_GPIO_OUT_CTRL:   std_logic_vector(NUM_CM_GPIO_OUT-1 downto 0) := (others => '0'); -- gpio-output enable

  constant DEV_ID:             std_logic_vector (31 downto 0) := x"53636C44";         -- "SclD"
  constant MEM_SIZE:           std_logic_vector (29 downto 0) := "00" & x"0000000";   -- required memory space for the custom modul

  constant SCALE_INT:          integer := 4;      -- scale factor: integer bits
  constant SCALE_FRAC:         integer := 4;      -- scale factor: fractional bits
  constant MAX_LLENGTH:        integer := 12;     -- max. picture line length = 2^MAX_LLENGTH
  constant MAX_LNUM:           integer := 12;     -- max. picture line number = 2^MAX_LNUM
  
  constant SCALE_FACT_DEFAULT: std_logic_vector(SCALE_INT+SCALE_FRAC-1 downto 0) := "0001" & "0000";
  constant PICX_DEFAULT:       std_logic_vector(MAX_LLENGTH-1 downto 0)          := x"500";
  constant PICY_DEFAULT:       std_logic_vector(MAX_LNUM-1 downto 0)             := x"400";
  
  constant MOD_DELAY:          integer := 15;     -- scale_d_cm module delay in clks
                         
  constant NOP:                std_logic_vector(2 downto 0) := "000";
  constant SOF:                std_logic_vector(2 downto 0) := "001";
  constant SOSLF:              std_logic_vector(2 downto 0) := "010";
  constant SOD:                std_logic_vector(2 downto 0) := "011";
  constant SOL:                std_logic_vector(2 downto 0) := "100";
  constant SOLL:               std_logic_vector(2 downto 0) := "101";
  constant EOL:                std_logic_vector(2 downto 0) := "110";
  constant ERR:                std_logic_vector(2 downto 0) := "111";

end package scale_d_cm_pkg;