-- ------------------------------------------------------------------------------
-- File Name         :  $RCSfile:  $
-- Author            :  $Author:  $
-- Revision          :  $Revision:  $
-- Last Modification :  $Date:  $
-- ------------------------------------------------------------------------------
--                                                 __________
--                                   _            / ________ \ 
--                                  | |          / /    ___ \ \ 
--                                  | |         / /    |__ \ \ \ 
--                                  | |        / /     __ \ \ \ \ 
--          ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
--         / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
--        | |      | | | |  ____| | | |            \ \____/ / / /
--        | |      | | | | |_____/  | |             \______/ / /
--        | |      | | | |________  | |________         ____/ /
--        |_|      |_|  \_________|  \_________|       |_____/
--
--                                   NEW ELECTRONIC TECHNOLOGY
--
-- ------------------------------------------------------------------------------
--
-- Copyright (C) 2016 NET GmbH All Rights Reserved.
--
-- This document, and its contents are the property of NET GmbH.
-- NET GmbH retains all rights to: this document, the document contents,
-- and the design techniques expressed by the document contents.
-- Neither this document, nor any of the information, nor any of the design
-- techniques expressed within may be, either in whole or in part: used,
-- reproduced, copied, stored, or conveyed to a third party by any means
-- without the express written consent of an authorized representative of
-- NET GmbH.
--
-- ------------------------------------------------------------------------------
-- Module     : hdr_cm_regif
-- Submodules : non
--
-- Purpose    : SystemBus Register-Interface
--
-- Creator    : a.kramer
-- ------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library work;
use work.scale_d_cm_pkg.all;

entity scale_d_cm_regif_e is
	
  port(
    -- Module Infrastructure Signals
    SysClkxCI:                  in std_logic;
    SysRstxRI:                  in std_logic;
    -- SystemBus Control Interface
  	SysAckxMO:					        out std_logic;
  	SysAddrxAI:					        in std_logic_vector(16 downto 0);
  	SysErrxMO:					        out std_logic;
  	SysRdDataxDO:               out std_logic_vector(31 downto 0);
    SysReqxMI:                  in std_logic;
    SysWrDataxDI:               in std_logic_vector(31 downto 0);
    SysWrEnxMI:                 in std_logic;
    -- Parameter
    param_mem_ptr_o:            out std_logic_vector(29 downto 0);
    param_mem_size_o:           out std_logic_vector(29 downto 0);
    param_mem_rdy_o:            out std_logic;
    param_picx_o:               out std_logic_vector(MAX_LLENGTH-1 downto 0);
    param_picy_o:               out std_logic_vector(MAX_LNUM-1 downto 0);
    param_picx_offs_o:          out std_logic_vector(MAX_LLENGTH-1 downto 0);
    param_picy_offs_o:          out std_logic_vector(MAX_LNUM-1 downto 0);
    param_scale_fact_o:         out std_logic_vector(SCALE_INT+SCALE_FRAC-1 downto 0);
    param_input_evt_o:          out std_logic;
    param_bypass_en_o:          out std_logic
    );	
end scale_d_cm_regif_e;


architecture scale_d_cm_regif_a of scale_d_cm_regif_e is

  signal ad_scale_id_sel:         std_logic;
  signal ad_mem_size_sel:         std_logic;
  signal ad_mem_ptr_sel:          std_logic;
  signal ad_mem_rdy_sel:          std_logic;
  signal ad_scale_conf_sel:       std_logic;
  signal ad_scale_picx_sel:       std_logic;
  signal ad_scale_picy_sel:       std_logic;
  signal ad_scale_picx_offs_sel:  std_logic;
  signal ad_scale_picy_offs_sel:  std_logic;

  signal ad_mem_ptr_evt:          std_logic;
  signal ad_mem_rdy_evt:          std_logic;
  signal ad_scale_conf_evt:       std_logic;
  signal ad_scale_picx_evt:       std_logic;
  signal ad_scale_picy_evt:       std_logic;
  signal ad_scale_picx_offs_evt:  std_logic;
  signal ad_scale_picy_offs_evt:  std_logic;
 
  signal rdg_rddata:              std_logic_vector (31 downto 0);
   
  signal rdg_scale_id:            std_logic_vector (31 downto 0);
  signal rdg_mem_size:            std_logic_vector (31 downto 0);
  signal rdg_mem_ptr:             std_logic_vector (31 downto 0);
  signal rdg_mem_rdy:             std_logic_vector (31 downto 0);
  signal rdg_scale_conf:          std_logic_vector (31 downto 0);
  signal rdg_scale_picx:          std_logic_vector (31 downto 0);
  signal rdg_scale_picy:          std_logic_vector (31 downto 0);
  signal rdg_scale_picx_offs:     std_logic_vector (31 downto 0);
  signal rdg_scale_picy_offs:     std_logic_vector (31 downto 0);
 
  signal param_mem_ptr_s:         std_logic_vector (29 downto 0);
  signal param_mem_rdy_s:         std_logic;
  signal param_picx_s:            std_logic_vector(MAX_LLENGTH-1 downto 0);
  signal param_picy_s:            std_logic_vector(MAX_LNUM-1 downto 0);
  signal param_picx_offs_s:       std_logic_vector(MAX_LLENGTH-1 downto 0);
  signal param_picy_offs_s:       std_logic_vector(MAX_LNUM-1 downto 0);
  signal param_scale_fact_s:      std_logic_vector(SCALE_INT+SCALE_FRAC-1 downto 0);
  signal param_input_evt_s:       std_logic;
  signal param_bypass_en_s:       std_logic;
  
  signal cg_write_evt:            std_logic;
  signal cg_read_evt:             std_logic;
  signal cg_read_ack:             std_logic;
  

begin
  
  ------------------------------------------------------------
  --address select
  ------------------------------------------------------------
  
  ad_scale_id_sel         <= '1'  when SysAddrxAI=('0' & x"0000") else '0';
  ad_scale_conf_sel       <= '1'  when SysAddrxAI=('0' & x"0004") else '0';
  ad_scale_picx_sel       <= '1'  when SysAddrxAI=('0' & x"0008") else '0';
  ad_scale_picy_sel       <= '1'  when SysAddrxAI=('0' & x"000c") else '0';
  ad_scale_picx_offs_sel  <= '1'  when SysAddrxAI=('0' & x"0010") else '0';
  ad_scale_picy_offs_sel  <= '1'  when SysAddrxAI=('0' & x"0014") else '0';
  ad_mem_ptr_sel          <= '1'  when SysAddrxAI=('1' & x"fe00") else '0'; 
  ad_mem_rdy_sel          <= '1'  when SysAddrxAI=('1' & x"fe04") else '0';
  ad_mem_size_sel         <= '1'  when SysAddrxAI=('1' & x"ff00") else '0';  
  
  
  ------------------------------------------------------------
  --address write-event
  ------------------------------------------------------------
  
  ad_mem_ptr_evt          <=  ad_mem_ptr_sel            and cg_write_evt;
  ad_mem_rdy_evt          <=  ad_mem_rdy_sel            and cg_write_evt;
  ad_scale_conf_evt       <=  ad_scale_conf_sel         and cg_write_evt;
  ad_scale_picx_evt       <=  ad_scale_picx_sel         and cg_write_evt;
  ad_scale_picy_evt       <=  ad_scale_picy_sel         and cg_write_evt;
  ad_scale_picx_offs_evt  <=  ad_scale_picx_offs_sel    and cg_write_evt;
  ad_scale_picy_offs_evt  <=  ad_scale_picy_offs_sel    and cg_write_evt;


  ------------------------------------------------------------
  --write data
  ------------------------------------------------------------ 
                                 
  init_wr: process (SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      param_mem_ptr_s           <= "00" & x"0000000";
      param_mem_rdy_s           <= '0';
      param_bypass_en_s         <= '0';
      param_input_evt_s         <= '0';
      param_scale_fact_s        <= SCALE_FACT_DEFAULT;
      param_picx_s              <= PICX_DEFAULT;
      param_picy_s              <= PICY_DEFAULT;
      param_picx_offs_s         <= (others => '0');
      param_picy_offs_s         <= (others => '0');
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (ad_mem_ptr_evt='1') then
        param_mem_ptr_s <= SysWrDataxDI(29 downto 0);
      end if;
      
      if (ad_mem_rdy_evt='1') then
        param_mem_rdy_s <= SysWrDataxDI(0);
      end if;
      
      if (ad_scale_conf_evt='1') then
        param_bypass_en_s  <= SysWrDataxDI(0);
        param_input_evt_s  <= SysWrDataxDI(1);
        param_scale_fact_s <= SysWrDataxDI(SCALE_INT+SCALE_FRAC+3 downto 4);
      else
        param_input_evt_s  <= '0';
      end if;
      
      if (ad_scale_picx_evt='1') then
        param_picx_s <= SysWrDataxDI(MAX_LLENGTH-1 downto 0);
      end if;
      
      if (ad_scale_picy_evt='1') then
        param_picy_s <= SysWrDataxDI(MAX_LNUM-1 downto 0);
      end if;
      
      if (ad_scale_picx_offs_evt='1') then
        param_picx_offs_s <= SysWrDataxDI(MAX_LLENGTH-1 downto 0);
      end if;
      
      if (ad_scale_picy_offs_evt='1') then
        param_picy_offs_s <= SysWrDataxDI(MAX_LNUM-1 downto 0);
      end if;
    end if;
  end process;
  
  
  param_mem_ptr_o          <= param_mem_ptr_s;
  param_mem_size_o         <= MEM_SIZE;
  param_mem_rdy_o          <= param_mem_rdy_s;
  param_picx_o             <= param_picx_s;
  param_picy_o             <= param_picy_s;
  param_picx_offs_o        <= param_picx_offs_s;
  param_picy_offs_o        <= param_picy_offs_s;
  param_scale_fact_o       <= param_scale_fact_s; 
  param_bypass_en_o        <= param_bypass_en_s;
  param_input_evt_o        <= param_input_evt_s;
  
  ------------------------------------------------------------
  --read data generation
  ------------------------------------------------------------
  
  rdg_rddata <= rdg_scale_id          when ad_scale_id_sel='1' else
                rdg_mem_size          when ad_mem_size_sel='1' else
                rdg_mem_ptr           when ad_mem_ptr_sel='1' else
                rdg_mem_rdy           when ad_mem_rdy_sel='1' else
                rdg_scale_conf        when ad_scale_conf_sel='1' else
                rdg_scale_picx        when ad_scale_picx_sel='1' else
                rdg_scale_picy        when ad_scale_picy_sel='1' else
                rdg_scale_picx_offs   when ad_scale_picx_offs_sel='1' else
                rdg_scale_picy_offs   when ad_scale_picy_offs_sel='1' else
                x"00000000";
              
  rdg_scale_id         <= DEV_ID;
  rdg_mem_size         <= "00" & MEM_SIZE;
  rdg_mem_ptr          <= "00" & param_mem_ptr_s;
  rdg_mem_rdy          <= x"0000000" & "000" & param_mem_rdy_s;
  rdg_scale_conf       <= std_logic_vector(to_unsigned(to_integer(unsigned(param_scale_fact_s)),28)) &
                          "000" & param_bypass_en_s;
  rdg_scale_picx       <= std_logic_vector(to_unsigned(to_integer(unsigned(param_picx_s)),32));
  rdg_scale_picy       <= std_logic_vector(to_unsigned(to_integer(unsigned(param_picy_s)),32));
  rdg_scale_picx_offs  <= std_logic_vector(to_unsigned(to_integer(unsigned(param_picx_offs_s)),32));
  rdg_scale_picy_offs  <= std_logic_vector(to_unsigned(to_integer(unsigned(param_picy_offs_s)),32));
  
   
  ------------------------------------------------------------
  --command generation
  ------------------------------------------------------------  
                              
  cg_write_evt <= SysReqxMI and SysWrEnxMI;
  cg_read_evt <= SysReqxMI and not SysWrEnxMI and not cg_read_ack;
  
  init_rd: process (SysClkxCI, SysRstxRI)
  begin   
    if (SysRstxRI='1') then
      cg_read_ack  <=  '0';
      SysRdDataxDO <= (others => '0');
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (cg_read_evt='1') then
        cg_read_ack <= '1';
        SysRdDataxDO  <= rdg_rddata;
      else
        cg_read_ack <= '0';
      end if;
    end if;
  end process;
  
  
  SysAckxMO  <= '1' when cg_write_evt='1' or cg_read_ack='1' else
                '0';
  
  SysErrxMO <= '0';
  
	
end scale_d_cm_regif_a;