-- ------------------------------------------------------------------------------
-- File Name         :  $RCSfile:  $
-- Author            :  $Author:  $
-- Revision          :  $Revision:  $
-- Last Modification :  $Date:  $
-- ------------------------------------------------------------------------------
--                                                 __________
--                                   _            / ________ \ 
--                                  | |          / /    ___ \ \ 
--                                  | |         / /    |__ \ \ \ 
--                                  | |        / /     __ \ \ \ \ 
--          ________     ________   | |____   /_/ __  /  \ \ \ \ \ 
--         / ______ \   / ______ \  |  ____|      \ \ \__/ / / / /
--        | |      | | | |  ____| | | |            \ \____/ / / /
--        | |      | | | | |_____/  | |             \______/ / /
--        | |      | | | |________  | |________         ____/ /
--        |_|      |_|  \_________|  \_________|       |_____/
--
--                                   NEW ELECTRONIC TECHNOLOGY
--
-- ------------------------------------------------------------------------------
--
-- Copyright (C) 2016 NET GmbH All Rights Reserved.
--
-- This document, and its contents are the property of NET GmbH.
-- NET GmbH retains all rights to: this document, the document contents,
-- and the design techniques expressed by the document contents.
-- Neither this document, nor any of the information, nor any of the design
-- techniques expressed within may be, either in whole or in part: used,
-- reproduced, copied, stored, or conveyed to a third party by any means
-- without the express written consent of an authorized representative of
-- NET GmbH.
--
-- ------------------------------------------------------------------------------
-- Module     : scale_d_cm_shadow_reg 
--
-- Purpose    : 
--
-- Creator    : a.kramer
-- ------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;

library work;
use work.scale_d_cm_pkg.all;


entity scale_d_cm_shadow_reg_e is	
  port(
    -- Module Infrastructure Signals
    SysClkxCI:               in std_logic;
    SysRstxRI:               in std_logic;
    -- ImageBus Receive Status
    ImgRxPixStatxSI:         in std_logic_vector(2 downto 0);
    ImgRxPixValxMI:          in std_logic;
    -- Shadow Parameter
    param_sr_picx_i:         in std_logic_vector(MAX_LLENGTH-1 downto 0);
    param_sr_picy_i:         in std_logic_vector(MAX_LNUM-1 downto 0);
    param_sr_picx_offs_i:    in std_logic_vector(MAX_LLENGTH-1 downto 0);
    param_sr_picy_offs_i:    in std_logic_vector(MAX_LNUM-1 downto 0);
    param_sr_scale_fact_i:   in std_logic_vector(SCALE_INT+SCALE_FRAC-1 downto 0);
    param_sr_input_evt_i:    in std_logic;
    param_sr_bypass_en_i:    in std_logic;
    -- Parameter
    param_picx_o:            out std_logic_vector(MAX_LLENGTH-1 downto 0);
    param_picy_o:            out std_logic_vector(MAX_LNUM-1 downto 0);
    param_picx_offs_o:       out std_logic_vector(MAX_LLENGTH-1 downto 0);
    param_picy_offs_o:       out std_logic_vector(MAX_LNUM-1 downto 0);
    param_scale_fact_o:      out std_logic_vector(SCALE_INT+SCALE_FRAC-1 downto 0);
    param_bypass_en_o:       out std_logic
    );  
end scale_d_cm_shadow_reg_e;



architecture scale_d_cm_shadow_reg_a of scale_d_cm_shadow_reg_e is
  
  signal last_line:              std_logic;
  signal frame_valid:            std_logic;
  signal frame_valid_buf:        std_logic;
  signal param_input_evt:        std_logic;
  signal param_input_evt_buf:    std_logic;
  
  signal soll_v:                 std_logic_vector(MOD_DELAY-1 downto 0);
  signal eol_v:                  std_logic_vector(MOD_DELAY-1 downto 0);
  
  
begin
  
  frame_valid <= '1' when (ImgRxPixValxMI='1' and ImgRxPixStatxSI=SOF) else
                 frame_valid_buf;
                 
  param_input_evt <= '1' when (param_sr_input_evt_i='1') else
                     param_input_evt_buf;

  
  ctl_reg: process(SysClkxCI, SysRstxRI)
  begin
    if (SysRstxRI='1') then
      soll_v               <= (others => '0');
      eol_v                <= (others => '0');
      last_line            <= '0';
      frame_valid_buf      <= '0';
      param_input_evt_buf  <= '0';
      param_bypass_en_o    <= '0';
      param_scale_fact_o   <= SCALE_FACT_DEFAULT;
      param_picx_o         <= PICX_DEFAULT;
      param_picy_o         <= PICY_DEFAULT;
      param_picx_offs_o    <= (others => '0');
      param_picy_offs_o    <= (others => '0');
      
    elsif (SysClkxCI'event and SysClkxCI='1') then
      if (ImgRxPixValxMI='1' and ImgRxPixStatxSI=SOLL) then
        soll_v <= soll_v(MOD_DELAY-2 downto 0) & '1';
      else  
        soll_v <= soll_v(MOD_DELAY-2 downto 0) & '0';
      end if;
      
      if (ImgRxPixValxMI='1' and ImgRxPixStatxSI=EOL) then
        eol_v <= eol_v(MOD_DELAY-2 downto 0) & '1';
      else  
        eol_v <= eol_v(MOD_DELAY-2 downto 0) & '0';
      end if;
      
      if (soll_v(MOD_DELAY-1)='1') then
        last_line <= '1';
      elsif (eol_v(MOD_DELAY-1)='1') then
        last_line <= '0';
      end if;
      
      if (eol_v(MOD_DELAY-1)='1' and last_line='1') then
        frame_valid_buf <= '0';
      else
        frame_valid_buf <= frame_valid;
      end if;
      
      if (frame_valid='0') then
        param_input_evt_buf <= '0';
      else
        param_input_evt_buf <= param_input_evt;
      end if;
      
      
      if (frame_valid='0') then
        param_bypass_en_o  <= param_sr_bypass_en_i;
      end if;
      
      if (frame_valid='0' and param_input_evt='1') then
        param_scale_fact_o <= param_sr_scale_fact_i;
        param_picx_o       <= param_sr_picx_i;
        param_picy_o       <= param_sr_picy_i;
        param_picx_offs_o  <= param_sr_picx_offs_i;
        param_picy_offs_o  <= param_sr_picy_offs_i;
      end if;
    end if;
  end process;

end scale_d_cm_shadow_reg_a;

