-------------------------------------------------------------------------------
-- Project    : CorSight2 - Custom Module
-- Title      : Package file for OpenCamera USER design
-------------------------------------------------------------------------------
-- Copyright (c) 2017 NET New Electronic Technology GmbH
-------------------------------------------------------------------------------
-- Revisions  : Date        Version  Author   Description
--              2017-10-20  1.0      Manfred  Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.CustomPkg.all;
use work.UserVerPkg.all;

package UserPkg is
    ----------------------------------------------------------------------------
    -- SystemBus Definitions
    ----------------------------------------------------------------------------
    constant cSYSMEN_ENABLE     : boolean := True;                          -- Enable SystemBus Memory Access
    constant cSYSMEM_START_ADDR : std_logic_vector(2 downto 0) := "100";    -- SystemBus Memory Start Address [eq. SysAddrxAI(16 downto 14)]

    -- USER control register template. Modify to suit application.
    type tUSR_CTRL_REG is record
        CtrlReg0  : std_logic;
        CtrlReg1  : std_logic_vector(3 downto 0);
    end record;

    constant cUSR_CTRL_DEFAULT : tUSR_CTRL_REG := (
        CtrlReg0  => '0',
        CtrlReg1  => (others => '0'));

    -- USER status register template. Modify to suit application.
    type tUSR_STAT_REG is record
        StatReg0  : std_logic;
        StatReg1  : std_logic_vector(3 downto 0);
    end record;

    constant cUSR_STAT_DEFAULT : tUSR_STAT_REG := ( 
        StatReg0  => '0',
        StatReg1  => (others => '0'));

    ----------------------------------------------------------------------------
    -- Version and Debug Definitions
    ----------------------------------------------------------------------------
    constant cVERSION_NUMBER   : integer := cCUSTOM_VERSION_NUM;        -- Set Custom Module version number
    constant cDEBUG_ENABLE     : boolean := cCUSTOM_DEBUG_ENABLE;       -- Enable/disable Custom Module debug mode
    constant cLOGIC_ENABLE     : boolean := cCUSTOM_ANALYZER_ENABLE;    -- Enable/disable Vivado Logic Analyzer
    constant cCHIPSCOPE_ENABLE : boolean := cCUSTOM_CHIPSCOPE_ENABLE;   -- Enable/disable ChipScopePro Analyzer
    constant cCSP_ICON_ENABLE  : boolean := cCUSTOM_CSP_ICON_ENABLE;    -- Enable/disable ICON module (JTAG mode only)
    constant cDEBUG_PORT_WIDTH : integer := cCUSTOM_DEBUG_PORT_WIDTH;   -- Width of ILA debug port

end UserPkg;
