################################################################################
# Project   : CorSight2 Rev1
# Title     : FPGA RTL timing constraints file
################################################################################
# Copyright (c) 2017 NET New Electronic Technology GmbH
################################################################################
# FPGA Family      - Artix7
# FPGA Part        - xc7a75t
# FPGA Package     - fgg484
# FPGA Speed Grade - -2
################################################################################

# Set FPGA voltage requirements
set_property CFGBVS VCCO [current_design]
set_property CONFIG_VOLTAGE 3.3 [current_design]
set_property CONFIG_MODE SPIx2 [current_design]

set_property BITSTREAM.CONFIG.CONFIGRATE 33     [current_design]
set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 2    [current_design]
set_property BITSTREAM.CONFIG.SPI_FALL_EDGE YES [current_design]
set_property BITSTREAM.CONFIG.SPI_32BIT_ADDR NO [current_design]
set_property BITSTREAM.CONFIG.UNUSEDPIN Pullup  [current_design]
set_property BITSTREAM.CONFIG.EXTMASTERCCLK_EN DISABLE [current_design]

#set_property BITSTREAM.CONFIG.SPI_OPCODE 8'h6B [current_design]
#set_property BITSTREAM.CONFIG.PERSIST YES      [current_design]
#set_property BITSTREAM.GENERAL.COMPRESS TRUE   [current_design]

###############################################################################
# Define FPGA oscillator clock
###############################################################################
# Define FPGA oscillator clock
create_clock -period 20.0 -name TS_OSC_CLK -waveform {0.0 10.0} [get_ports OscClkxCI]
set_input_jitter [get_clocks -of_objects [get_ports OscClkxCI]] 0.10
set_propagated_clock OscClkxCI

set_input_delay -clock [get_clocks TS_OSC_CLK] -min -add_delay 1.0 [get_ports {Gpio_TTLInxMI[*] Gpio_OptoInxMI[*] Cam0_SRstxRIO Cam0_IO_CtrlxMIO[*] Cam0_MC_SDataxDIO Cam0_I2C_SDataxDIO Cam0_JTAG_TDIxDI}]
set_input_delay -clock [get_clocks TS_OSC_CLK] -max -add_delay 5.0 [get_ports {Gpio_TTLInxMI[*] Gpio_OptoInxMI[*] Cam0_SRstxRIO Cam0_IO_CtrlxMIO[*] Cam0_MC_SDataxDIO Cam0_I2C_SDataxDIO Cam0_JTAG_TDIxDI}]

set_output_delay -clock TS_OSC_CLK -min -1.0 [get_ports {Cam0_SRstxRIO Cam0_IO_CtrlxMIO[*] Cam0_MC_SClkxMO Cam0_MC_SDataxDIO Cam0_I2C_SClkxMO Cam0_I2C_SDataxDIO Cam0_SPI_SLxMO Cam0_SPI_SClkxCO Cam0_SPI_DataxDO Cam0_JTAG_TCKxCO Cam0_JTAG_TMSxMO Cam0_JTAG_TDOxDO}]
set_output_delay -clock TS_OSC_CLK -max  1.0 [get_ports {Cam0_SRstxRIO Cam0_IO_CtrlxMIO[*] Cam0_MC_SClkxMO Cam0_MC_SDataxDIO Cam0_I2C_SClkxMO Cam0_I2C_SDataxDIO Cam0_SPI_SLxMO Cam0_SPI_SClkxCO Cam0_SPI_DataxDO Cam0_JTAG_TCKxCO Cam0_JTAG_TMSxMO Cam0_JTAG_TDOxDO}]
set_output_delay -clock TS_OSC_CLK -min -1.0 [get_ports {Gpio_TTLOutxMO[*] Gpio_OptoOutxMO[*] P55Volt_EnxMO P38Volt_EnxMO FlashStrb_EnxMO[*]}]
set_output_delay -clock TS_OSC_CLK -max  1.0 [get_ports {Gpio_TTLOutxMO[*] Gpio_OptoOutxMO[*] P55Volt_EnxMO P38Volt_EnxMO FlashStrb_EnxMO[*]}]

# set_property PHASESHIFT_MODE WAVEFORM [get_cells -hierarchical -filter {NAME =~ */ClockGenInst/MMCME2_ADV_Inst}]

###############################################################################
# Define Sensor clock
###############################################################################
create_clock -period 6.666 -name TS_CAM_CLK [get_ports Cam0_PixClkxCI]
set_input_jitter [get_clocks -of_objects [get_ports Cam0_PixClkxCI]] 0.4

set_clock_groups -asynchronous -group [get_clocks -include_generated_clocks TS_OSC_CLK] -group [get_clocks -include_generated_clocks TS_CAM_CLK]
set_clock_groups -asynchronous -group [get_clocks -include_generated_clocks TS_OSC_CLK] -group [get_clocks -include_generated_clocks userclk1]

set_input_delay -clock [get_clocks TS_CAM_CLK] -min 2.5 [get_ports {Cam0_HSyncxMI Cam0_VSyncxMI Cam0_DataxDI[*]}]
set_input_delay -clock [get_clocks TS_CAM_CLK] -max 3.0 [get_ports {Cam0_HSyncxMI Cam0_VSyncxMI Cam0_DataxDI[*]}]

###############################################################################
# PCIe Timing Constraints
###############################################################################
# Define PCI-Express clock
create_clock -period 10.0 -name TS_PCI_CLK -waveform {0.0 5.0} [get_ports Pcie_RefClkPxCI]
set_property LOC IBUFDS_GTE2_X0Y2 [get_cells -hierarchical -filter {NAME =~ */PcieInst/PcieCoreInst/PcieRefClkBuf}]

set_false_path -to [get_pins -hierarchical -filter {NAME =~ */pipe_clock_i/pclk_i1_bufgctrl.pclk_i1/S0}]
set_false_path -to [get_pins -hierarchical -filter {NAME =~ */pipe_clock_i/pclk_i1_bufgctrl.pclk_i1/S1}]

create_generated_clock -name clk_125mhz_x0y0 [get_pins -hierarchical -filter {NAME =~ */pipe_clock_i/mmcm_i/CLKOUT0}]
create_generated_clock -name clk_250mhz_x0y0 [get_pins -hierarchical -filter {NAME =~ */pipe_clock_i/mmcm_i/CLKOUT1}]
create_generated_clock -name clk_125mhz_mux_x0y0 -source [get_pins -hierarchical -filter {NAME =~ */pipe_clock_i/pclk_i1_bufgctrl.pclk_i1/I0}] -divide_by 1 [get_pins -hierarchical -filter {NAME =~ */pipe_clock_i/pclk_i1_bufgctrl.pclk_i1/O}]
create_generated_clock -name clk_250mhz_mux_x0y0 -source [get_pins -hierarchical -filter {NAME =~ */pipe_clock_i/pclk_i1_bufgctrl.pclk_i1/I1}] -divide_by 1 -add -master_clock clk_250mhz_x0y0 [get_pins -hierarchical -filter {NAME =~ */pipe_clock_i/pclk_i1_bufgctrl.pclk_i1/O}]
set_clock_groups -name pcieclkmux -physically_exclusive -group clk_125mhz_mux_x0y0 -group clk_250mhz_mux_x0y0

###############################################################################
# Define cross clock data paths
###############################################################################
set_max_delay -datapath_only -from [get_cells -hierarchical -filter {NAME =~ */DualMcbInst/*/McbLvsbAckxM_reg}]        -to [get_cells -hierarchical -filter {NAME =~ */DualMcbInst/*/McbSyncAckxM_reg[0]}] 2.0
set_max_delay -datapath_only -from [get_cells -hierarchical -filter {NAME =~ */DualMcbInst/*/McbLvsbDatxD_reg[*]}]     -to [get_cells -hierarchical -filter {NAME =~ */DualMcbInst/*/McbRdDatxD_reg[*]}] 2.0
set_max_delay -datapath_only -from [get_cells -hierarchical -filter {NAME =~ */DualMcbInst/*/McbLvsbErrxM_reg}]        -to [get_cells -hierarchical -filter {NAME =~ */DualMcbInst/*/McbSyncErrxM_reg[0]}] 2.0
set_max_delay -datapath_only -from [get_pins  -hierarchical -filter {NAME =~ */DualMcbInst/*/McbAppDoneIdxM_reg[*]/C}] -to [get_pins  -hierarchical -filter {NAME =~ */DualMcbInst/*/McbStrWrDoneIdxM_reg[*]/D}] 8.0

set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieCoreInst/PcieBlockX1Gen.PcieBlockX1Inst/PcieBlock_x1_i/U0/inst/pcie_top_i/pcie_7x_i/pcie_block_i/USERCLK2}] -to [get_pins -hierarchical -filter {NAME =~ */DmaCtrlInst/DmaPcieFifoComp/A7_PcktFifoGen.A7_DmaPcktFifo/U0/inst_fifo_gen/gconvfifo.rf/grf.rf/gntv_or_sync_fifo.mem/gdm.dm_gen.dm/RAM_reg_0_63_6_7/RAMB/I}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieCoreInst/PcieBlockX1Gen.PcieBlockX1Inst/PcieBlock_x1_i/U0/inst/pcie_top_i/pcie_7x_i/pcie_block_i/USERCLK2}] -to [get_pins -hierarchical -filter {NAME =~ */DmaCtrlInst/DmaAddrGenComp/DmaRdPayLdSyncxM_reg[*]/D}] 8.0

set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */ClockGenInst/SysRstLoop[4].SysRstInst/C}] -to [get_pins -hierarchical -filter {NAME =~ */TimerInst/LvsbSlaveComp/ExtiCfgRegxM_reg[*][TimeConstxM][*]/R}] 8.0

set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieRxComp/TlpRxMemWrxM_reg/C}]         -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/LvsbMstInst/LvsbWeSyncxM_reg[0]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieRxComp/PcieCoreStbxM_reg*/C}]       -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/LvsbMstInst/LvsbStbSyncxM_reg[0]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieRxComp/PcieCoreDWEnxM_reg[*]/C}]    -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/LvsbMstInst/LvsbDWEnSyncxM_reg[0][*]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieRxComp/PcieCoreEoBxM_reg*/C}]       -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/LvsbMstInst/LvsbEoBSyncxM_reg[0]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieRxComp/PcieCoreAddrxA_reg[*]/C}]    -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/LvsbMstInst/LvsbAddrSyncxA_reg[*]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieRxComp/PcieCoreWrDataxD_reg[*]/C}]  -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/LvsbMstInst/LvsbWrDataSyncxD_reg[*]/D}] 8.0

set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieTxComp/SyncByteEnIrqxM_reg/C}]      -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieTxComp/SyncBeIReqxM_reg[0]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieTxComp/TrnTxEofIReqxM_reg[*]/C}]    -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieTxComp/SyncEofIReqxM_reg[0][*]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieTxComp/CfgIPendCntxM_reg[*]*/C}]    -to [get_pins -hierarchical -filter {NAME =~ */LvsbCtrInst/LvsbIrqProc/IrqSyncPendxM_reg[0]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieTxComp/CfgIReqErrorxM_reg*/C}]      -to [get_pins -hierarchical -filter {NAME =~ */LvsbCtrInst/LvsbIrqProc/IrqSyncErrxM_reg[0]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */LvsbCtrInst/LvsbIrqProc/IrqGlobalxM_reg/C}]      -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieTxComp/LvsbIReqSyncxM_reg[0]/D}] 8.0

set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/LvsbMstInst/PcieCoreDonexM_reg/C}]      -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieRxComp/LvsbDoneSyncxM_reg[0]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/LvsbMstInst/PcieCoreAckxM_reg/C}]       -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieRxComp/LvsbAckSyncxM_reg[0]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */PcieInst/LvsbMstInst/PcieCoreRdDataxD_reg[*]/C}] -to [get_pins -hierarchical -filter {NAME =~ */PcieInst/PcieRxComp/LvsbRdDataSyncxM_reg[*]/D}] 8.0

set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */lvsbInterconn_1/LvsbSlvAddrxA*/C}] -to [get_pins -hierarchical -filter {NAME =~ */DualMcbInst/*/McbLvsbDatxD_reg[*]/CE}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */lvsbInterconn_1/LvsbSlvAddrxA*/C}] -to [get_pins -hierarchical -filter {NAME =~ */DualMcbInst/*/McbLvsbDatxD_reg[*]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */lvsbInterconn_1/LvsbSlvAddrxA*/C}] -to [get_pins -hierarchical -filter {NAME =~ */DualMcbInst/*/McbLvsbErrxM_reg/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */lvsbInterconn_1/LvsbSlvAddrxA*/C}] -to [get_pins -hierarchical -filter {NAME =~ */DualMcbInst/*/LvsbMcbAddrxA_reg[*]/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */lvsbInterconn_1/LvsbSlvAddrxA*/C}] -to [get_pins -hierarchical -filter {NAME =~ */DualMcbInst/*/LvsbMcbReqxM_reg/D}] 8.0
set_max_delay -datapath_only -from [get_pins -hierarchical -filter {NAME =~ */lvsbInterconn_1/LvsbSlvAddrxA*/C}] -to [get_pins -hierarchical -filter {NAME =~ */DualMcbInst/*/McbSyncReqxM_reg/D}] 8.0

###############################################################################
# Set False Paths (TIG)
###############################################################################
set_false_path -from [get_ports SysResetxRIL]
set_false_path -from [get_ports BoardRevisionxMIL[*]]
set_false_path -from [get_ports BoardConfigxMI[*]]
set_false_path -from [get_ports TestPointxMIO[*]]
set_false_path -from [get_ports Gpio_TTLInxMI[*]]
set_false_path -from [get_ports Gpio_OptoInxMI[*]]
set_false_path -from [get_ports Cam0_JTAG_TDIxDI]
set_false_path -from [get_ports Cam0_SRstxRIO]
set_false_path -from [get_ports Cam0_IO_CtrlxMIO[*]]
set_false_path -from [get_ports Cam0_I2C_SDataxDIO]
set_false_path -from [get_ports Cam0_MC_SDataxDIO]

set_false_path -to [get_ports Cam0_SRstxRIO]
set_false_path -to [get_ports Cam0_IO_CtrlxMIO[*]]
set_false_path -to [get_ports Cam0_I2C_SClkxMO]
set_false_path -to [get_ports Cam0_MC_SClkxMO]
set_false_path -to [get_ports Cam0_I2C_SDataxDIO]
set_false_path -to [get_ports Cam0_MC_SDataxDIO]
set_false_path -to [get_ports Cam0_JTAG_TCKxCO]
set_false_path -to [get_ports Cam0_JTAG_TDOxDO]
set_false_path -to [get_ports Cam0_JTAG_TMSxMO]
set_false_path -to [get_ports Cam0_SPI_SClkxCO]
set_false_path -to [get_ports Cam0_SPI_DataxDO]
set_false_path -to [get_ports Cam0_SPI_SLxMO]
set_false_path -to [get_ports FlashStrb_EnxMO[*]]
set_false_path -to [get_ports Gpio_OptoOutxMO[*]]
set_false_path -to [get_ports Gpio_TTLOutxMO[*]]
set_false_path -to [get_ports P55Volt_EnxMO]
set_false_path -to [get_ports P38Volt_EnxMO]
set_false_path -to [get_ports TestPointxMIO[*]]

set_false_path -from [get_pins -hierarchical -filter {NAME =~ */DmaCtrlInst/DmaPcieFifoComp/DmaFifoRstxR_reg/C}]

###############################################################################
# Define PAD input/output delays
###############################################################################
set_property IOB TRUE [get_ports Cam0_HSyncxMI]
set_property IOB TRUE [get_ports Cam0_VSyncxMI]
set_property IOB TRUE [get_ports Cam0_DataxDI[*]]
set_property IOB TRUE [get_ports Cam0_SRstxRIO]
set_property IOB TRUE [get_ports Cam0_IO_CtrlxMIO[*]]
set_property IOB TRUE [get_ports Cam0_MC_SClkxMO]
set_property IOB TRUE [get_ports Cam0_MC_SDataxDIO]
set_property IOB TRUE [get_ports Cam0_I2C_SClkxMO]
set_property IOB TRUE [get_ports Cam0_I2C_SDataxDIO]
set_property IOB TRUE [get_ports Cam0_SPI_SLxMO]
set_property IOB TRUE [get_ports Cam0_SPI_SClkxCO]
set_property IOB TRUE [get_ports Cam0_SPI_DataxDO]
set_property IOB TRUE [get_ports Cam0_JTAG_TCKxCO]
set_property IOB TRUE [get_ports Cam0_JTAG_TMSxMO]
set_property IOB TRUE [get_ports Cam0_JTAG_TDOxDO]
set_property IOB TRUE [get_ports Cam0_JTAG_TDIxDI]

set_property IOB TRUE [get_ports Gpio_TTLInxMI[*]]
set_property IOB TRUE [get_ports Gpio_TTLOutxMO[*]]
set_property IOB TRUE [get_ports Gpio_OptoInxMI[*]]
set_property IOB TRUE [get_ports Gpio_OptoOutxMO[*]]

set_property IOB TRUE [get_ports P55Volt_EnxMO]
set_property IOB TRUE [get_ports P38Volt_EnxMO]
set_property IOB TRUE [get_ports FlashStrb_EnxMO[*]]

set_property IOB TRUE [get_ports BoardRevisionxMIL[*]]
set_property IOB TRUE [get_ports BoardConfigxMI[*]]
set_property IOB TRUE [get_ports TestPointxMIO[*]]
