################################################################################
# Project   : CorSight2
# Title     : Vivado FPGA compilation command script
# Copyright (c) 2017 NET New Electronic Technology GmbH
################################################################################
# This File is generated automatically - DO NOT MODIFY
################################################################################

set ProjectDir  [get_property DIRECTORY [current_project]]
set ProjectName [get_property NAME      [current_project]]
set DesignName  design_1
cd $ProjectDir

set_property XPM_LIBRARIES {XPM_CDC XPM_MEMORY XPM_FIFO} [current_project]
set_property target_language VHDL [current_project]

puts "Reading Debug Project file..."
source $ProjectDir/DebugProject.tcl
puts "Reading Custom Module Project file..."
source $ProjectDir/CustomProject.tcl

# Add source files to the Vivado project...
puts "Reading FPGA Project file..."
source $ProjectDir/FpgaProject.tcl

# Set FPGA compile options
set_property strategy Flow_PerfOptimized_High [get_runs synth_1]
set_property STEPS.SYNTH_DESIGN.ARGS.RETIMING true [get_runs synth_1]
set_property strategy Performance_ExplorePostRoutePhysOpt [get_runs impl_1]
set_property STEPS.PHYS_OPT_DESIGN.IS_ENABLED true [get_runs impl_1]
set_property STEPS.WRITE_BITSTREAM.ARGS.BIN_FILE true [get_runs impl_1]
set_param logicopt.enableBUFGinsertCLK 0
set_property used_in_synthesis      true [get_files FpgaConstraints.xdc]
set_property used_in_implementation true [get_files FpgaConstraints.xdc]
set_property used_in_synthesis      true [get_files FpgaPinout.xdc]
set_property used_in_implementation true [get_files FpgaPinout.xdc]

# Suppress the following non-critical Vivado warning messages:
set_msg_config -suppress -id {Common 17-206}
set_msg_config -suppress -id {Synth 8-115}
set_msg_config -suppress -id {Synth 8-350}
set_msg_config -suppress -id {Synth 8-567}
set_msg_config -suppress -id {Synth 8-689}
set_msg_config -suppress -id {Synth 8-3295}
set_msg_config -suppress -id {Synth 8-3321}
set_msg_config -suppress -id {Synth 8-3331}
set_msg_config -suppress -id {Synth 8-3332}
set_msg_config -suppress -id {Synth 8-3848}
set_msg_config -suppress -id {Synth 8-3917}
set_msg_config -suppress -id {Synth 8-3919}
set_msg_config -suppress -id {Synth 8-3936}
set_msg_config -suppress -id {Synth 8-5396}
set_msg_config -suppress -id {Synth 8-5788}
set_msg_config -suppress -id {Synth 8-5974}
set_msg_config -suppress -id {Synth 8-6014}
set_msg_config -suppress -id {Designutils 20-1567}
set_msg_config -suppress -id {Power 33-332}
set_msg_config -suppress -id {Timing 38-436}
set_msg_config -suppress -id {IP_Flow 19-519}
set_msg_config -suppress -id {DRC CHECK-3}
set_msg_config -suppress -id {DRC REQP-1709}
set_msg_config -suppress -id {DRC REQP-1839}
set_msg_config -suppress -id {DRC REQP-1840}
set_msg_config -suppress -id {DRC RTSTAT-10}
set_msg_config -suppress -id {DRC BUFC-1}
set_msg_config -suppress -id {DRC DPIP-1}
set_msg_config -suppress -id {DRC DPOP-1}
set_msg_config -suppress -id {DRC PDRC-153}

# ======================================================================================================

# Run synthesis - Initialize script variables.
set ElapsedTime  1
set SynthError   0
set ImplError    0
set SynthTimeOut 30
set BitFname     $ProjectDir/$ProjectName.runs/impl_1/${DesignName}_wrapper.bit

# Generate target data for all IP cores.
generate_target all [get_ips]

# Launch the FPGA Synthesis process.
launch_runs synth_1 -jobs 4

# Wait until synthesis completes or a timeout occurs.
while 1 {
    set SynthStatus   [get_property STATUS [get_runs synth_1]]
    set SynthProgress [get_property PROGRESS [get_runs synth_1]]

    if {$SynthStatus == "synth_design Complete!" && $SynthProgress == "100%"} {
        puts "Vivado Synthesis completed successfully..."
        break
    } else {
        after 60000
        puts "Vivado Synthesis in progress - Elapsed Time: $ElapsedTime min"
        if {$ElapsedTime < $SynthTimeOut} {
            incr ElapsedTime
        } else {
            puts "ERROR: Vivado Synthesis Timeout - Synthesis Aborted"
            set SynthError 1
            break
        }
    }
}

# ======================================================================================================

# If no synthesis timeout error has occurred, run FPGA implementation
if {$SynthError == 0} {
    set ElapsedTime 1
    set ImplTimeOut 60

    # Launch the FPGA Implementation process.
    launch_runs impl_1 -to_step write_bitstream -jobs 4

    while 1 {
        set ImplStatus   [get_property STATUS [get_runs impl_1]]
        set ImplProgress [get_property PROGRESS [get_runs impl_1]]

        if {$ImplStatus == "write_bitstream Complete!" && $ImplProgress == "100%"} {
            after 30000
            if {[file exist $CS2_BIN_FNAME] > 0} {
                puts "FPGA bit-stream file generated successfully..."
                open_run impl_1
                check_timing -file Cs2_Check_Timing.rpt
                report_timing_summary -report_unconstrained -file Cs2_Timing_Summary.rpt
                report_timing_summary -setup -hold -no_detailed_paths -file Cs2_Timing_Setup_Hold.rpt
                report_bus_skew -file Cs2_Bus_Skew.rpt
                report_cdc -detail -file Cs2_CDC_Detail.rpt
                report_cdc -summary -file Cs2_CDC_Summary.rpt
                report_utilization -file Cs2_Utilization_Summary.rpt
                report_utilization -hierarchical -file Cs2_Utilization_Hierarchical.rpt
                if {[file isdirectory $CS2_RELEASE_DIR] == 0} {
                    file mkdir $CS2_RELEASE_DIR
                }
                file copy -force $CS2_BIN_FNAME $CS2_RELEASE_FNAME
            } else {
                puts "ERROR - FPGA binary bit stream file not found - Implementation Aborted"
                set ImplError 1
            }
            break
        } else {
            after 60000
            puts "$ImplStatus - Elapsed Time: $ElapsedTime min"
            if {$ElapsedTime < $ImplTimeOut} {
                incr ElapsedTime
            } else {
                puts "ERROR: Vivado Implementation Timeout - Implementation Aborted"
                set ImplError 1
                break
            }
        }
    }
}

if {$SynthError == 0 && $ImplError == 0} {
    # Create a variable containing a timing summary report
    set Cs2TimingReport [report_timing_summary -no_header -no_detailed_paths -return_string]

    # Search the timing summary report to detect if timing has been met
    if {[string match -nocase {*timing constraints are met*} $Cs2TimingReport]} {
        append LogTxt "No Timing Errors detected - FPGA Implementation completed successfully...\n"
    } else {
        append LogTxt "ERROR - Timing Errors have been detected - FPGA Implementation failed...\n"
        append LogTxt "        Please check the Timing Summary Report in $CS2_COMPILE_DIR\n"
    }

} else { 
    if {$SynthError > 0} {
        append LogTxt "ERROR - Vivado Synthesis Errors detected - FPGA Synthesis aborted...\n"
        append LogTxt "        Please check the Synthesis Report in $CS2_COMPILE_DIR\n"
    }

    if {$ImplError > 0} {
        append LogTxt "ERROR - Vivado Implementation Errors detected - FPGA Implementation aborted...\n"
        append LogTxt "        Please check the Implementation Report in $CS2_COMPILE_DIR\n"
    }
}

set SysTime [clock seconds]
append LogTxt "FPGA compilation completed on [clock format $SysTime -format %d/%m/%Y] at [clock format $SysTime -format %H:%M:%S]:\n\n"
append LogTxt "-------------------------------------------------------------------------------\n\n"

if {$VIVADO_MODE == "CLOSE"} {
    close_design
    stop_gui
}

if {$SynthError == 0 && $ImplError == 0} {
    return ok
} else {
    return error
}

